from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.args import feature_view__client_pb2 as _feature_view__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetFeaturesBatchParameters(_message.Message):
    __slots__ = ["feature_service_id", "feature_service_name", "metadata_options", "request_data", "workspace_name"]
    FEATURE_SERVICE_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVICE_NAME_FIELD_NUMBER: ClassVar[int]
    METADATA_OPTIONS_FIELD_NUMBER: ClassVar[int]
    REQUEST_DATA_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_NAME_FIELD_NUMBER: ClassVar[int]
    feature_service_id: str
    feature_service_name: str
    metadata_options: MetadataOptions
    request_data: _containers.RepeatedCompositeFieldContainer[GetFeaturesBatchRequestData]
    workspace_name: str
    def __init__(self, feature_service_id: Optional[str] = ..., feature_service_name: Optional[str] = ..., workspace_name: Optional[str] = ..., request_data: Optional[Iterable[Union[GetFeaturesBatchRequestData, Mapping]]] = ..., metadata_options: Optional[Union[MetadataOptions, Mapping]] = ...) -> None: ...

class GetFeaturesBatchRequest(_message.Message):
    __slots__ = ["params"]
    PARAMS_FIELD_NUMBER: ClassVar[int]
    params: GetFeaturesBatchParameters
    def __init__(self, params: Optional[Union[GetFeaturesBatchParameters, Mapping]] = ...) -> None: ...

class GetFeaturesBatchRequestData(_message.Message):
    __slots__ = ["join_key_map", "request_context_map"]
    class JoinKeyMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    class RequestContextMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    JOIN_KEY_MAP_FIELD_NUMBER: ClassVar[int]
    REQUEST_CONTEXT_MAP_FIELD_NUMBER: ClassVar[int]
    join_key_map: _containers.MessageMap[str, _struct_pb2.Value]
    request_context_map: _containers.MessageMap[str, _struct_pb2.Value]
    def __init__(self, join_key_map: Optional[Mapping[str, _struct_pb2.Value]] = ..., request_context_map: Optional[Mapping[str, _struct_pb2.Value]] = ...) -> None: ...

class GetFeaturesParameters(_message.Message):
    __slots__ = ["allow_partial_results", "feature_package_id", "feature_package_name", "feature_service_id", "feature_service_name", "feature_view_id", "feature_view_name", "isCallerBatch", "join_key_map", "metadata_options", "request_context_map", "request_options", "workspace_name"]
    class JoinKeyMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    class RequestContextMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(self, key: Optional[str] = ..., value: Optional[Union[_struct_pb2.Value, Mapping]] = ...) -> None: ...
    ALLOW_PARTIAL_RESULTS_FIELD_NUMBER: ClassVar[int]
    FEATURE_PACKAGE_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_PACKAGE_NAME_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVICE_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVICE_NAME_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    ISCALLERBATCH_FIELD_NUMBER: ClassVar[int]
    JOIN_KEY_MAP_FIELD_NUMBER: ClassVar[int]
    METADATA_OPTIONS_FIELD_NUMBER: ClassVar[int]
    REQUEST_CONTEXT_MAP_FIELD_NUMBER: ClassVar[int]
    REQUEST_OPTIONS_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_NAME_FIELD_NUMBER: ClassVar[int]
    allow_partial_results: bool
    feature_package_id: str
    feature_package_name: str
    feature_service_id: str
    feature_service_name: str
    feature_view_id: str
    feature_view_name: str
    isCallerBatch: bool
    join_key_map: _containers.MessageMap[str, _struct_pb2.Value]
    metadata_options: MetadataOptions
    request_context_map: _containers.MessageMap[str, _struct_pb2.Value]
    request_options: RequestOptions
    workspace_name: str
    def __init__(self, feature_service_id: Optional[str] = ..., feature_service_name: Optional[str] = ..., feature_package_id: Optional[str] = ..., feature_package_name: Optional[str] = ..., feature_view_id: Optional[str] = ..., feature_view_name: Optional[str] = ..., workspace_name: Optional[str] = ..., join_key_map: Optional[Mapping[str, _struct_pb2.Value]] = ..., request_context_map: Optional[Mapping[str, _struct_pb2.Value]] = ..., metadata_options: Optional[Union[MetadataOptions, Mapping]] = ..., allow_partial_results: bool = ..., request_options: Optional[Union[RequestOptions, Mapping]] = ..., isCallerBatch: bool = ...) -> None: ...

class GetFeaturesRequest(_message.Message):
    __slots__ = ["params"]
    PARAMS_FIELD_NUMBER: ClassVar[int]
    params: GetFeaturesParameters
    def __init__(self, params: Optional[Union[GetFeaturesParameters, Mapping]] = ...) -> None: ...

class MetadataOptions(_message.Message):
    __slots__ = ["include_data_types", "include_effective_times", "include_feature_descriptions", "include_feature_tags", "include_names", "include_serving_status", "include_slo_info", "include_types"]
    INCLUDE_DATA_TYPES_FIELD_NUMBER: ClassVar[int]
    INCLUDE_EFFECTIVE_TIMES_FIELD_NUMBER: ClassVar[int]
    INCLUDE_FEATURE_DESCRIPTIONS_FIELD_NUMBER: ClassVar[int]
    INCLUDE_FEATURE_TAGS_FIELD_NUMBER: ClassVar[int]
    INCLUDE_NAMES_FIELD_NUMBER: ClassVar[int]
    INCLUDE_SERVING_STATUS_FIELD_NUMBER: ClassVar[int]
    INCLUDE_SLO_INFO_FIELD_NUMBER: ClassVar[int]
    INCLUDE_TYPES_FIELD_NUMBER: ClassVar[int]
    include_data_types: bool
    include_effective_times: bool
    include_feature_descriptions: bool
    include_feature_tags: bool
    include_names: bool
    include_serving_status: bool
    include_slo_info: bool
    include_types: bool
    def __init__(self, include_names: bool = ..., include_effective_times: bool = ..., include_types: bool = ..., include_data_types: bool = ..., include_slo_info: bool = ..., include_serving_status: bool = ..., include_feature_descriptions: bool = ..., include_feature_tags: bool = ...) -> None: ...

class RequestOptions(_message.Message):
    __slots__ = ["aggregation_leading_edge", "high_watermark_override", "read_from_cache", "write_to_cache"]
    AGGREGATION_LEADING_EDGE_FIELD_NUMBER: ClassVar[int]
    HIGH_WATERMARK_OVERRIDE_FIELD_NUMBER: ClassVar[int]
    READ_FROM_CACHE_FIELD_NUMBER: ClassVar[int]
    WRITE_TO_CACHE_FIELD_NUMBER: ClassVar[int]
    aggregation_leading_edge: _feature_view__client_pb2.AggregationLeadingEdge
    high_watermark_override: _timestamp_pb2.Timestamp
    read_from_cache: bool
    write_to_cache: bool
    def __init__(self, read_from_cache: bool = ..., write_to_cache: bool = ..., aggregation_leading_edge: Optional[Union[_feature_view__client_pb2.AggregationLeadingEdge, str]] = ..., high_watermark_override: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/snowflake/location__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-tecton_proto/snowflake/location__client.proto\x12\x16tecton_proto.snowflake\"`\n\rStageLocation\x12\x1c\n\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x1d\n\nstage_name\x18\x02 \x01(\tR\tstageName\x12\x12\n\x04path\x18\x03 \x01(\tR\x04pathB\x18\n\x14\x63om.tecton.snowflakeP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.snowflake.location__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.tecton.snowflakeP\001'
  _STAGELOCATION._serialized_start=73
  _STAGELOCATION._serialized_end=169
# @@protoc_insertion_point(module_scope)



from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class FcoLocator(_message.Message):
    __slots__ = ["id", "name", "workspace"]
    ID_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    id: _id__client_pb2.Id
    name: str
    workspace: str
    def __init__(self, name: Optional[str] = ..., id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., workspace: Optional[str] = ...) -> None: ...

class IdFcoLocator(_message.Message):
    __slots__ = ["id", "workspace", "workspace_state_id"]
    ID_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    id: _id__client_pb2.Id
    workspace: str
    workspace_state_id: _id__client_pb2.Id
    def __init__(self, id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ...) -> None: ...



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateWorkspaceDirectoryRequest(_message.Message):
    __slots__ = ["path"]
    PATH_FIELD_NUMBER: ClassVar[int]
    path: str
    def __init__(self, path: Optional[str] = ...) -> None: ...

class GetWorkspaceObjectStatusRequest(_message.Message):
    __slots__ = ["path"]
    PATH_FIELD_NUMBER: ClassVar[int]
    path: str
    def __init__(self, path: Optional[str] = ...) -> None: ...

class GetWorkspaceObjectStatusResponse(_message.Message):
    __slots__ = ["created_at", "language", "modified_at", "object_id", "object_type", "path", "size"]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    LANGUAGE_FIELD_NUMBER: ClassVar[int]
    MODIFIED_AT_FIELD_NUMBER: ClassVar[int]
    OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    OBJECT_TYPE_FIELD_NUMBER: ClassVar[int]
    PATH_FIELD_NUMBER: ClassVar[int]
    SIZE_FIELD_NUMBER: ClassVar[int]
    created_at: int
    language: str
    modified_at: int
    object_id: int
    object_type: str
    path: str
    size: int
    def __init__(self, object_type: Optional[str] = ..., path: Optional[str] = ..., language: Optional[str] = ..., created_at: Optional[int] = ..., modified_at: Optional[int] = ..., object_id: Optional[int] = ..., size: Optional[int] = ...) -> None: ...

class ListWorkspaceRequest(_message.Message):
    __slots__ = ["path"]
    PATH_FIELD_NUMBER: ClassVar[int]
    path: str
    def __init__(self, path: Optional[str] = ...) -> None: ...

class ListWorkspaceResponse(_message.Message):
    __slots__ = ["objects"]
    OBJECTS_FIELD_NUMBER: ClassVar[int]
    objects: _containers.RepeatedCompositeFieldContainer[ObjectInfo]
    def __init__(self, objects: Optional[Iterable[Union[ObjectInfo, Mapping]]] = ...) -> None: ...

class ObjectInfo(_message.Message):
    __slots__ = ["language", "object_id", "object_type", "path"]
    LANGUAGE_FIELD_NUMBER: ClassVar[int]
    OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    OBJECT_TYPE_FIELD_NUMBER: ClassVar[int]
    PATH_FIELD_NUMBER: ClassVar[int]
    language: str
    object_id: int
    object_type: str
    path: str
    def __init__(self, object_type: Optional[str] = ..., object_id: Optional[int] = ..., path: Optional[str] = ..., language: Optional[str] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.dataobs import expectation__client_pb2 as _expectation__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor
RESULT_ERROR: ExpectationResultEnum
RESULT_FAILED: ExpectationResultEnum
RESULT_PASSED: ExpectationResultEnum
RESULT_UNKNOWN: ExpectationResultEnum

class ExpectationResult(_message.Message):
    __slots__ = ["feature_expectation_metadata", "feature_interval_end_time", "feature_interval_start_time", "feature_package_id", "feature_view_name", "metric_expectation_metadata", "result", "result_id", "validation_job_id", "validation_time", "workspace"]
    FEATURE_EXPECTATION_METADATA_FIELD_NUMBER: ClassVar[int]
    FEATURE_INTERVAL_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_INTERVAL_START_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_PACKAGE_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    METRIC_EXPECTATION_METADATA_FIELD_NUMBER: ClassVar[int]
    RESULT_FIELD_NUMBER: ClassVar[int]
    RESULT_ID_FIELD_NUMBER: ClassVar[int]
    VALIDATION_JOB_ID_FIELD_NUMBER: ClassVar[int]
    VALIDATION_TIME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    feature_expectation_metadata: FeatureExpectationMetadata
    feature_interval_end_time: _timestamp_pb2.Timestamp
    feature_interval_start_time: _timestamp_pb2.Timestamp
    feature_package_id: _id__client_pb2.Id
    feature_view_name: str
    metric_expectation_metadata: MetricExpectationMetadata
    result: ExpectationResultEnum
    result_id: _id__client_pb2.Id
    validation_job_id: _id__client_pb2.Id
    validation_time: _timestamp_pb2.Timestamp
    workspace: str
    def __init__(self, validation_job_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., workspace: Optional[str] = ..., feature_view_name: Optional[str] = ..., feature_package_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., validation_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_interval_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_interval_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_expectation_metadata: Optional[Union[FeatureExpectationMetadata, Mapping]] = ..., metric_expectation_metadata: Optional[Union[MetricExpectationMetadata, Mapping]] = ..., result: Optional[Union[ExpectationResultEnum, str]] = ..., result_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ...) -> None: ...

class ExpectationResultSummary(_message.Message):
    __slots__ = ["expectation_name", "summary"]
    EXPECTATION_NAME_FIELD_NUMBER: ClassVar[int]
    SUMMARY_FIELD_NUMBER: ClassVar[int]
    expectation_name: str
    summary: ResultSummary
    def __init__(self, expectation_name: Optional[str] = ..., summary: Optional[Union[ResultSummary, Mapping]] = ...) -> None: ...

class FeatureExpectationMetadata(_message.Message):
    __slots__ = ["alert_msg", "expectation", "failed_join_key_samples", "failure_percentage"]
    ALERT_MSG_FIELD_NUMBER: ClassVar[int]
    EXPECTATION_FIELD_NUMBER: ClassVar[int]
    FAILED_JOIN_KEY_SAMPLES_FIELD_NUMBER: ClassVar[int]
    FAILURE_PERCENTAGE_FIELD_NUMBER: ClassVar[int]
    alert_msg: str
    expectation: _expectation__client_pb2.FeatureExpectation
    failed_join_key_samples: _containers.RepeatedScalarFieldContainer[str]
    failure_percentage: float
    def __init__(self, expectation: Optional[Union[_expectation__client_pb2.FeatureExpectation, Mapping]] = ..., alert_msg: Optional[str] = ..., failure_percentage: Optional[float] = ..., failed_join_key_samples: Optional[Iterable[str]] = ...) -> None: ...

class FeatureViewResultSummary(_message.Message):
    __slots__ = ["expectation_summary", "feature_view_name", "summary"]
    EXPECTATION_SUMMARY_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    SUMMARY_FIELD_NUMBER: ClassVar[int]
    expectation_summary: _containers.RepeatedCompositeFieldContainer[ExpectationResultSummary]
    feature_view_name: str
    summary: ResultSummary
    def __init__(self, feature_view_name: Optional[str] = ..., summary: Optional[Union[ResultSummary, Mapping]] = ..., expectation_summary: Optional[Iterable[Union[ExpectationResultSummary, Mapping]]] = ...) -> None: ...

class MetricExpectationMetadata(_message.Message):
    __slots__ = ["alert_msg", "expectation", "param_values"]
    class ParamValue(_message.Message):
        __slots__ = ["actual_value", "interval_start_time", "metric_name"]
        ACTUAL_VALUE_FIELD_NUMBER: ClassVar[int]
        INTERVAL_START_TIME_FIELD_NUMBER: ClassVar[int]
        METRIC_NAME_FIELD_NUMBER: ClassVar[int]
        actual_value: str
        interval_start_time: _timestamp_pb2.Timestamp
        metric_name: str
        def __init__(self, metric_name: Optional[str] = ..., actual_value: Optional[str] = ..., interval_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...
    ALERT_MSG_FIELD_NUMBER: ClassVar[int]
    EXPECTATION_FIELD_NUMBER: ClassVar[int]
    PARAM_VALUES_FIELD_NUMBER: ClassVar[int]
    alert_msg: str
    expectation: _expectation__client_pb2.MetricExpectation
    param_values: _containers.RepeatedCompositeFieldContainer[MetricExpectationMetadata.ParamValue]
    def __init__(self, expectation: Optional[Union[_expectation__client_pb2.MetricExpectation, Mapping]] = ..., alert_msg: Optional[str] = ..., param_values: Optional[Iterable[Union[MetricExpectationMetadata.ParamValue, Mapping]]] = ...) -> None: ...

class ResultSummary(_message.Message):
    __slots__ = ["error", "failed", "passed", "unknown"]
    ERROR_FIELD_NUMBER: ClassVar[int]
    FAILED_FIELD_NUMBER: ClassVar[int]
    PASSED_FIELD_NUMBER: ClassVar[int]
    UNKNOWN_FIELD_NUMBER: ClassVar[int]
    error: int
    failed: int
    passed: int
    unknown: int
    def __init__(self, passed: Optional[int] = ..., failed: Optional[int] = ..., error: Optional[int] = ..., unknown: Optional[int] = ...) -> None: ...

class WorkspaceResultSummary(_message.Message):
    __slots__ = ["feature_view_summary", "summary", "workspace"]
    FEATURE_VIEW_SUMMARY_FIELD_NUMBER: ClassVar[int]
    SUMMARY_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    feature_view_summary: _containers.RepeatedCompositeFieldContainer[FeatureViewResultSummary]
    summary: ResultSummary
    workspace: str
    def __init__(self, workspace: Optional[str] = ..., summary: Optional[Union[ResultSummary, Mapping]] = ..., feature_view_summary: Optional[Iterable[Union[FeatureViewResultSummary, Mapping]]] = ...) -> None: ...

class ExpectationResultEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

AVG_LENGTH: MetricType
AVG_VALUE: MetricType
COUNT_DISTINCT: MetricType
COUNT_NULLS: MetricType
COUNT_ROWS: MetricType
COUNT_ZEROS: MetricType
DESCRIPTOR: _descriptor.FileDescriptor
MAX_VALUE: MetricType
METRIC_STATUS_AVAILABLE: MetricStatus
METRIC_STATUS_NO_MATERIALIZATION: MetricStatus
METRIC_STATUS_UNAVAILABLE: MetricStatus
METRIC_STATUS_UNKNOWN: MetricStatus
METRIC_TYPE_UNKNOWN: MetricType
MIN_VALUE: MetricType
STDDEV_SAMPLE: MetricType
VAR_SAMPLE: MetricType

class FeatureMetric(_message.Message):
    __slots__ = ["column_name", "feature_name", "metric_type"]
    COLUMN_NAME_FIELD_NUMBER: ClassVar[int]
    FEATURE_NAME_FIELD_NUMBER: ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: ClassVar[int]
    column_name: str
    feature_name: str
    metric_type: MetricType
    def __init__(self, metric_type: Optional[Union[MetricType, str]] = ..., feature_name: Optional[str] = ..., column_name: Optional[str] = ...) -> None: ...

class Metric(_message.Message):
    __slots__ = ["creation_time", "expression", "input_column_names", "interval", "last_update_time", "name", "window"]
    CREATION_TIME_FIELD_NUMBER: ClassVar[int]
    EXPRESSION_FIELD_NUMBER: ClassVar[int]
    INPUT_COLUMN_NAMES_FIELD_NUMBER: ClassVar[int]
    INTERVAL_FIELD_NUMBER: ClassVar[int]
    LAST_UPDATE_TIME_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    WINDOW_FIELD_NUMBER: ClassVar[int]
    creation_time: _timestamp_pb2.Timestamp
    expression: str
    input_column_names: _containers.RepeatedScalarFieldContainer[str]
    interval: _duration_pb2.Duration
    last_update_time: _timestamp_pb2.Timestamp
    name: str
    window: _duration_pb2.Duration
    def __init__(self, name: Optional[str] = ..., expression: Optional[str] = ..., window: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., interval: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., input_column_names: Optional[Iterable[str]] = ..., creation_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., last_update_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class MetricDataPoint(_message.Message):
    __slots__ = ["interval_start_time", "materialization_run_id", "materialization_task_attempt_url", "metric_status", "metric_values"]
    INTERVAL_START_TIME_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_RUN_ID_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_TASK_ATTEMPT_URL_FIELD_NUMBER: ClassVar[int]
    METRIC_STATUS_FIELD_NUMBER: ClassVar[int]
    METRIC_VALUES_FIELD_NUMBER: ClassVar[int]
    interval_start_time: _timestamp_pb2.Timestamp
    materialization_run_id: str
    materialization_task_attempt_url: str
    metric_status: MetricStatus
    metric_values: _containers.RepeatedCompositeFieldContainer[MetricValue]
    def __init__(self, interval_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., metric_values: Optional[Iterable[Union[MetricValue, Mapping]]] = ..., materialization_run_id: Optional[str] = ..., materialization_task_attempt_url: Optional[str] = ..., metric_status: Optional[Union[MetricStatus, str]] = ...) -> None: ...

class MetricValue(_message.Message):
    __slots__ = ["feature_name", "value"]
    FEATURE_NAME_FIELD_NUMBER: ClassVar[int]
    VALUE_FIELD_NUMBER: ClassVar[int]
    feature_name: str
    value: str
    def __init__(self, feature_name: Optional[str] = ..., value: Optional[str] = ...) -> None: ...

class MetricType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MetricStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from datetime import timedelta
from typing import Dict
from typing import Optional
from typing import Union

import pandas

from tecton_core import specs
from tecton_core.compute_mode import ComputeMode
from tecton_core.feature_definition_wrapper import FeatureDefinitionWrapper
from tecton_core.feature_set_config import FeatureSetConfig
from tecton_core.query.node_interface import NodeRef


@dataclass
class GetFeaturesForEventsParams:
    fco: Union[specs.FeatureServiceSpec, FeatureDefinitionWrapper]
    events: Union[pandas.DataFrame, str]
    compute_mode: ComputeMode
    timestamp_key: Optional[str] = None
    from_source: Optional[bool] = None
    mock_data_sources: Optional[Dict[str, NodeRef]] = None
    # Only used for feature service
    feature_set_config: Optional[FeatureSetConfig] = None

    @property
    def join_keys(self):
        if self.feature_set_config:
            return self.feature_set_config.join_keys

        assert isinstance(self.fco, FeatureDefinitionWrapper)
        return self.fco.join_keys


@dataclass
class GetFeaturesInRangeParams:
    fco: FeatureDefinitionWrapper
    start_time: datetime
    end_time: datetime
    compute_mode: ComputeMode
    entities: Optional[pandas.DataFrame, str] = None
    max_lookback: Optional[timedelta] = None
    from_source: Optional[bool] = None
    mock_data_sources: Optional[Dict[str, NodeRef]] = None

    @property
    def join_keys(self):
        return self.fco.join_keys



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Optional

DESCRIPTOR: _descriptor.FileDescriptor

class SnowflakeCredentials(_message.Message):
    __slots__ = ["password_secret_name", "private_key_alias", "user"]
    PASSWORD_SECRET_NAME_FIELD_NUMBER: ClassVar[int]
    PRIVATE_KEY_ALIAS_FIELD_NUMBER: ClassVar[int]
    USER_FIELD_NUMBER: ClassVar[int]
    password_secret_name: str
    private_key_alias: str
    user: str
    def __init__(self, user: Optional[str] = ..., password_secret_name: Optional[str] = ..., private_key_alias: Optional[str] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.auditlog import metadata__client_pb2 as _metadata__client_pb2
from tecton_proto.auth import principal__client_pb2 as _principal__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class PrincipalGroup(_message.Message):
    __slots__ = ["created_at", "created_by", "description", "id", "idp_mapping_names", "is_membership_editable", "name", "updated_at"]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    CREATED_BY_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    IDP_MAPPING_NAMES_FIELD_NUMBER: ClassVar[int]
    ID_FIELD_NUMBER: ClassVar[int]
    IS_MEMBERSHIP_EDITABLE_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    UPDATED_AT_FIELD_NUMBER: ClassVar[int]
    created_at: _timestamp_pb2.Timestamp
    created_by: _principal__client_pb2.Principal
    description: str
    id: str
    idp_mapping_names: _containers.RepeatedScalarFieldContainer[str]
    is_membership_editable: bool
    name: str
    updated_at: _timestamp_pb2.Timestamp
    def __init__(self, id: Optional[str] = ..., name: Optional[str] = ..., description: Optional[str] = ..., idp_mapping_names: Optional[Iterable[str]] = ..., is_membership_editable: bool = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., created_by: Optional[Union[_principal__client_pb2.Principal, Mapping]] = ..., updated_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...



import re
from collections import defaultdict
from pathlib import Path
from typing import Dict
from typing import List
from typing import Optional

import click

from tecton.cli.command import TectonCommand
from tecton.cli.printer import safe_print
from tecton.cli.upgrade_utils import BatchFeatureViewGuidance
from tecton.cli.upgrade_utils import DataSourceGuidance
from tecton.cli.upgrade_utils import EntityGuidance
from tecton.cli.upgrade_utils import FeatureTableGuidance
from tecton.cli.upgrade_utils import OnDemandFeatureViewGuidance
from tecton.cli.upgrade_utils import StreamFeatureViewGuidance
from tecton.framework.base_tecton_object import BaseTectonObject
from tecton.repo_utils import collect_top_level_objects
from tecton_core import conf
from tecton_core import repo_file_handler
from tecton_core.feature_definition_wrapper import FrameworkVersion


OBJECTS_TO_MIGRATE = {
    "BatchSource",
    "StreamSource",
    "PushSource",
    "Entity",
    "BatchFeatureView",
    "StreamFeatureView",
    "OnDemandFeatureView",
    "FeatureTable",
}

# if an object to migrate (Entity) is not in OBJECT_TO_MIGRATE_SECTION_OVERRIDE_NAME,
# it is stored as "Entity"
OBJECT_TO_MIGRATE_SECTION_OVERRIDE_NAME = {
    "BatchSource": "DataSource",
    "StreamSource": "DataSource",
    "PushSource": "DataSource",
}

OBJECT_TO_MIGRATE_TO_GUIDANCE_STEPS = [
    ("DataSource", DataSourceGuidance),
    ("Entity", EntityGuidance),
    ("BatchFeatureView", BatchFeatureViewGuidance),
    ("StreamFeatureView", StreamFeatureViewGuidance),
    ("OnDemandFeatureView", OnDemandFeatureViewGuidance),
    ("FeatureTable", FeatureTableGuidance),
]


def _fco_to_pretty_print(fco: str, count: int):
    if fco == "Entity":
        if count == 1:
            return "Entity"
        return "Entities"
    else:
        # converts camelcase to words with space
        # Ex: camelCase -> camel case
        pretty_string = re.sub(r"(?<!^)(?=[A-Z])", " ", fco)
        if count == 1:
            return pretty_string
        return f"{pretty_string}s"


def process_files(py_files: List[Path], repo_root: Path) -> Dict[str, List[Dict[str, any]]]:
    all_objects = defaultdict(list)

    # First, collect all FCOs
    fcos = collect_top_level_objects(py_files, repo_root, True)
    for tecton_object in fcos:
        fco_type = type(tecton_object).__name__
        key = (
            OBJECT_TO_MIGRATE_SECTION_OVERRIDE_NAME.get(fco_type, fco_type)
            if fco_type in OBJECTS_TO_MIGRATE
            else "Other"
        )
        all_objects[key].append(tecton_object)

    # Next, go through all files and collect non-FCO imports
    # TODO: handling "Other" imports and getting precision on them is kind of hard
    # How do you differentiate between an explicit transformation and one that is not explicit but baked into a FV?
    # Probably not worth all the extra effort; just say "Convert everything else".
    # Will revisit later.
    # with printer.safe_yaspin(yaspin.spinners.Spinners.earth, text="Analyzing feature repository modules") as sp:
    #     for file_path in py_files:
    #         sp.text = f"Processing {file_path.relative_to(repo_root)}"
    #         fcos = collect_non_fco_imports(file_path, repo_root, py_files)
    #         all_objects["Other"].extend(fcos)
    return dict(all_objects)


def print_migration_progress(all_objects: Dict[str, BaseTectonObject]):
    print("\nMigration Progress:")
    steps = [
        ("Data Sources", "DataSource"),
        ("Entities", "Entity"),
        ("Batch Feature Views", "BatchFeatureView"),
        ("Stream Feature Views", "StreamFeatureView"),
        ("On-Demand Feature Views", "OnDemandFeatureView"),
        ("Feature Tables", "FeatureTable"),
    ]

    for i, (step_name, obj_type) in enumerate(steps, 1):
        objects = all_objects.get(obj_type, [])
        total = len(objects)
        migrated = sum(1 for obj in objects if obj._framework_version == FrameworkVersion.FWV6)
        status = "✅" if migrated == total else "🟨" if migrated > 0 else "🚫"
        migrate_text = (
            f"{migrated}/{total} {_fco_to_pretty_print(obj_type, migrated)} migrated"
            if total > 0
            else f"No {_fco_to_pretty_print(obj_type, 0)} to migrate"
        )
        safe_print(f"  Step {i}: Migrate {step_name} - {status} ({migrate_text}).")
    safe_print(f"  Step {len(steps) + 1}: Migrate all other imports from `tecton.v09_compat` to `tecton`.")


def print_interactive_migration_guidance(all_objects: Dict[str, List[Dict[str, any]]], repo_root: str) -> None:
    step = 1
    index = 0

    while step <= len(OBJECT_TO_MIGRATE_TO_GUIDANCE_STEPS):
        obj_type, guildanceBuilder = OBJECT_TO_MIGRATE_TO_GUIDANCE_STEPS[step - 1]
        objects_to_migrate = sorted(
            [obj for obj in all_objects.get(obj_type, []) if obj._framework_version != FrameworkVersion.FWV6],
            key=lambda obj: (obj._source_info.source_filename, int(obj._source_info.source_lineno)),
        )

        if not objects_to_migrate:
            step += 1
            continue

        if index == 0:
            safe_print(f"\nStep {step}: Migrate {_fco_to_pretty_print(obj_type, len(objects_to_migrate))}.")
            safe_print(
                f"\nDetected {len(objects_to_migrate)} {_fco_to_pretty_print(obj_type, len(objects_to_migrate))} in need of an upgrade:\n"
            )

        safe_print(
            f"\nNumber {index + 1} of {len(objects_to_migrate)} {_fco_to_pretty_print(obj_type, len(objects_to_migrate))}:\n"
        )
        guildanceBuilder(objects_to_migrate[index], repo_root).print_guidance()

        # Display options
        if index > 0 and index < len(objects_to_migrate) - 1:
            options = "[n]ext, [p]revious, [q]uit"
        elif index == 0:
            options = "[n]ext, [q]uit"
        elif index == len(objects_to_migrate) - 1:
            options = "[p]revious, [q]uit"
        else:
            msg = f"Invalid index {index}."
            raise ValueError(msg)

        user_input = click.prompt(f"Continue migration? {options}", type=str).lower()

        if user_input == "n":
            if index < len(objects_to_migrate) - 1:
                index += 1
            else:
                safe_print(
                    f"Step {step}: Migrate {_fco_to_pretty_print(obj_type, len(objects_to_migrate))} complete. Run `tecton plan` to ensure these objects were correctly migrated (you may apply this plan to commit your progress). Then run `tecton upgrade` to continue the migration."
                )
                return
        elif user_input == "p" and index > 0:
            index -= 1
        elif user_input == "q":
            safe_print("Process quit by user.")
            return
        elif user_input == "p" and index == 0 and step > 1:
            step -= 1
            index = len(objects_to_migrate) - 1
        else:
            safe_print(f"Unknown input {user_input}. Try again.")


def print_non_interactive_migration_guidance(all_objects: Dict[str, List[Dict[str, any]]], repo_root: str) -> None:
    for step, (obj_type, GuidanceBuilder) in enumerate(
        OBJECT_TO_MIGRATE_TO_GUIDANCE_STEPS,
        1,
    ):
        objects_to_migrate = sorted(
            [obj for obj in all_objects.get(obj_type, []) if obj._framework_version != FrameworkVersion.FWV6],
            key=lambda obj: (obj._source_info.source_filename, int(obj._source_info.source_lineno)),
        )
        if bool(objects_to_migrate):
            safe_print(f"\nStep {step}: Migrate {_fco_to_pretty_print(obj_type, len(objects_to_migrate))}.")
            safe_print(
                f"\nDetected {len(objects_to_migrate)} {_fco_to_pretty_print(obj_type, len(objects_to_migrate))} in need of an upgrade:\n"
            )
            for obj in objects_to_migrate:
                GuidanceBuilder(obj, repo_root).print_guidance()
            break
    safe_print(
        "Run `tecton plan` when complete to ensure there are no destructive changes. Then run `tecton upgrade` to get the updated set of instructions."
    )


def print_migration_guidance(all_objects: Dict[str, List[Dict[str, any]]], repo_root: str, interactive: bool) -> None:
    if interactive:
        print_interactive_migration_guidance(all_objects=all_objects, repo_root=repo_root)
    else:
        print_non_interactive_migration_guidance(all_objects=all_objects, repo_root=repo_root)


@click.command(cls=TectonCommand)
@click.argument(
    "path",
    required=False,
    default=None,
    type=click.Path(exists=True, dir_okay=True, path_type=Path, readable=True),
)
@click.option("--non-interactive", is_flag=True, help="Run in non-interactive mode.")
def upgrade(path: Optional[Path] = None, non_interactive: bool = False):
    """Analyze Tecton objects in the feature repository and provide migration guidance.

    If PATH is provided, analyze the specified file or directory.
    If PATH is not provided, analyze all Python files in the repository root.
    """
    repo_file_handler.ensure_prepare_repo()
    repo_root = repo_file_handler.repo_root()
    repo_files = repo_file_handler.repo_files()
    py_files = [p for p in repo_files if p.suffix == ".py"]

    if path:
        full_path = path.resolve()
        if full_path.is_file():
            py_files = [full_path]
        elif full_path.is_dir():
            py_files = [p for p in full_path.rglob("*.py") if p.is_file()]
        else:
            print(f"Error: {full_path} is not a valid file or directory.")
            return

    interactive = not non_interactive
    # Skip `validate` so that it is safe to execute 0.9 objects
    with conf._temporary_set("TECTON_SKIP_OBJECT_VALIDATION", True), conf._temporary_set(
        "TECTON_REQUIRE_SCHEMA", False
    ):
        all_objects = process_files(py_files, Path(repo_root))
        print_migration_progress(all_objects)
        print_migration_guidance(all_objects, repo_root, interactive)



from dataclasses import dataclass
from datetime import datetime
from typing import Optional


@dataclass
class FilterContext:
    """`FilterContext` is passed as an argument to the data source function when `supports_time_filtering` is set to `True`. Using these parameters enables optimized query patterns for improved performance:

     - The method `<data source>.get_dataframe()` can be invoked with the arguments `start_time` or `end_time`.
     - When defining a Feature View, a FilteredSource can be paired with a Data Source. The Feature View will then pass FilterContext into the Data Source Function

    Note that Data Source Functions are expected to implement their own filtering logic.

    ```python
    from tecton import spark_batch_config
    from pyspark.sql.functions import col

    @spark_batch_config(supports_time_filtering=True)
    def hive_data_source_function(spark, filter_context):
        spark.sql(f"USE {hive_db_name}")
        df = spark.table(user_hive_table)
        ts_column = "timestamp"

        # Data Source Function handles its own filtering logic here
        if filter_context:
            if filter_context.start_time:
                df = df.where(col(ts_column) >= filter_context.start_time)
            if filter_context.end_time:
                df = df.where(col(ts_column) < filter_context.end_time)
        return df
    ```

    :param start_time: If specified, data source will only include items with timestamp column >= start_time
    :param end_time: If specified, data source will only include items with timestamp column < end_time


    """

    start_time: Optional[datetime]
    end_time: Optional[datetime]



from tecton_proto.auth import service__client_pb2 as _service__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Mapping, Optional, Text, Union

DESCRIPTOR: _descriptor.FileDescriptor

class HttpRequest(_message.Message):
    __slots__ = ["body", "contentLength", "contentType", "method", "url"]
    BODY_FIELD_NUMBER: ClassVar[int]
    CONTENTLENGTH_FIELD_NUMBER: ClassVar[int]
    CONTENTTYPE_FIELD_NUMBER: ClassVar[int]
    METHOD_FIELD_NUMBER: ClassVar[int]
    URL_FIELD_NUMBER: ClassVar[int]
    body: bytes
    contentLength: int
    contentType: str
    method: str
    url: str
    def __init__(self, method: Optional[str] = ..., url: Optional[str] = ..., body: Optional[bytes] = ..., contentType: Optional[str] = ..., contentLength: Optional[int] = ...) -> None: ...

class HttpResponseHeaders(_message.Message):
    __slots__ = ["contentLength", "contentType", "status"]
    CONTENTLENGTH_FIELD_NUMBER: ClassVar[int]
    CONTENTTYPE_FIELD_NUMBER: ClassVar[int]
    STATUS_FIELD_NUMBER: ClassVar[int]
    contentLength: int
    contentType: str
    status: int
    def __init__(self, status: Optional[int] = ..., contentType: Optional[str] = ..., contentLength: Optional[int] = ...) -> None: ...

class HttpResponsePiece(_message.Message):
    __slots__ = ["body_piece", "headers"]
    BODY_PIECE_FIELD_NUMBER: ClassVar[int]
    HEADERS_FIELD_NUMBER: ClassVar[int]
    body_piece: bytes
    headers: HttpResponseHeaders
    def __init__(self, headers: Optional[Union[HttpResponseHeaders, Mapping]] = ..., body_piece: Optional[bytes] = ...) -> None: ...



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class LifetimeWindow(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class RelativeTimeWindow(_message.Message):
    __slots__ = ["window_end", "window_start"]
    WINDOW_END_FIELD_NUMBER: ClassVar[int]
    WINDOW_START_FIELD_NUMBER: ClassVar[int]
    window_end: _duration_pb2.Duration
    window_start: _duration_pb2.Duration
    def __init__(self, window_start: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., window_end: Optional[Union[_duration_pb2.Duration, Mapping]] = ...) -> None: ...

class TimeWindow(_message.Message):
    __slots__ = ["lifetime_window", "relative_time_window", "time_window_series"]
    LIFETIME_WINDOW_FIELD_NUMBER: ClassVar[int]
    RELATIVE_TIME_WINDOW_FIELD_NUMBER: ClassVar[int]
    TIME_WINDOW_SERIES_FIELD_NUMBER: ClassVar[int]
    lifetime_window: LifetimeWindow
    relative_time_window: RelativeTimeWindow
    time_window_series: TimeWindowSeries
    def __init__(self, relative_time_window: Optional[Union[RelativeTimeWindow, Mapping]] = ..., lifetime_window: Optional[Union[LifetimeWindow, Mapping]] = ..., time_window_series: Optional[Union[TimeWindowSeries, Mapping]] = ...) -> None: ...

class TimeWindowSeries(_message.Message):
    __slots__ = ["time_windows"]
    TIME_WINDOWS_FIELD_NUMBER: ClassVar[int]
    time_windows: _containers.RepeatedCompositeFieldContainer[RelativeTimeWindow]
    def __init__(self, time_windows: Optional[Iterable[Union[RelativeTimeWindow, Mapping]]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor
ONBOARDING_STATUS_COMPLETED: OnboardingStatusEnum
ONBOARDING_STATUS_INCOMPLETE: OnboardingStatusEnum
ONBOARDING_STATUS_UNSPECIFIED: OnboardingStatusEnum
TASK_STATUS_FAILED: DataPlatformSetupTaskStatusEnum
TASK_STATUS_NOT_STARTED: DataPlatformSetupTaskStatusEnum
TASK_STATUS_RUNNING: DataPlatformSetupTaskStatusEnum
TASK_STATUS_SUCCEEDED: DataPlatformSetupTaskStatusEnum
TASK_STATUS_UNKNOWN: DataPlatformSetupTaskStatusEnum

class DataPlatformSetupTaskStatus(_message.Message):
    __slots__ = ["details", "error_message", "task_display_name", "task_status"]
    DETAILS_FIELD_NUMBER: ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: ClassVar[int]
    TASK_DISPLAY_NAME_FIELD_NUMBER: ClassVar[int]
    TASK_STATUS_FIELD_NUMBER: ClassVar[int]
    details: str
    error_message: str
    task_display_name: str
    task_status: DataPlatformSetupTaskStatusEnum
    def __init__(self, task_display_name: Optional[str] = ..., task_status: Optional[Union[DataPlatformSetupTaskStatusEnum, str]] = ..., error_message: Optional[str] = ..., details: Optional[str] = ...) -> None: ...

class OnboardingStatus(_message.Message):
    __slots__ = ["status", "user_id"]
    STATUS_FIELD_NUMBER: ClassVar[int]
    USER_ID_FIELD_NUMBER: ClassVar[int]
    status: OnboardingStatusEnum
    user_id: str
    def __init__(self, user_id: Optional[str] = ..., status: Optional[Union[OnboardingStatusEnum, str]] = ...) -> None: ...

class OnboardingStatusEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class DataPlatformSetupTaskStatusEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



import ast
import glob
import os
import sys
from pathlib import Path
from typing import Callable
from typing import List

from tecton.cli import printer
from tecton_core.errors import TectonValidationError
from tecton_proto.modelartifactservice.model_artifact_service__client_pb2 import ModelType


def error_and_exit(message: str):
    error_message = f"⛔ {message}"
    printer.safe_print(error_message, file=sys.stderr)
    sys.exit(1)


def _find_function_and_validate(objects: List[ast.FunctionDef], name: str, condition: Callable[[int], bool]):
    find_object = [obj for obj in objects if obj.name == name]
    if not condition(len(find_object)):
        error_and_exit(f"Exactly one `{name}` function should be found in the model file.")


def validate(model_file_path: Path, artifact_files: List[str], archive_root_path: Path) -> None:
    if not os.path.isfile(model_file_path):
        error_and_exit(f"File {model_file_path} defined in ModelConfig does not exist.")

    invalid_file_paths = []
    for file_path in artifact_files:
        files = glob.glob(str(archive_root_path / file_path))
        if len(files) == 0:
            invalid_file_paths.append(file_path)
        for file in files:
            if Path(archive_root_path).resolve() not in Path(file).resolve().parents:
                error_and_exit(
                    f"All `artifact_files` must be within directory containing model config file. {Path(archive_root_path).resolve()} : {Path(file).resolve().parents}"
                )
    if len(invalid_file_paths) > 0:
        error_and_exit(f"File paths: {','.join(invalid_file_paths)} defined in ModelConfig does not match any files.")

    model_file_text = model_file_path.read_text()
    ast_module = ast.parse(model_file_text)
    functions = [obj for obj in ast_module.body if type(obj) == ast.FunctionDef]

    _find_function_and_validate(functions, "load_context", lambda x: x == 1)
    _find_function_and_validate(functions, "preprocessor", lambda x: x <= 1)
    _find_function_and_validate(functions, "postprocessor", lambda x: x <= 1)


def model_type_string_to_enum(model_type: str) -> ModelType:
    if model_type == "pytorch":
        return ModelType.PYTORCH
    msg = f"Invalid Model Type: {model_type}"
    raise TectonValidationError(msg)



from datetime import datetime
from datetime import timedelta
from unittest import TestCase

from tecton_core import conf
from tecton_core.data_processing_utils import split_range


class SplitRangeTest(TestCase):
    def test_split_range_even(self):
        conf.set("DUCKDB_RANGE_SPLIT_COUNT", 3)
        range_start = datetime(2024, 1, 1, 0, 0, 0)
        range_end = datetime(2024, 1, 1, 3, 0, 0)

        split_ranges = split_range(range_start, range_end, timedelta(hours=1))

        expected_ranges = [
            (datetime(2024, 1, 1, 0, 0, 0), datetime(2024, 1, 1, 1, 0, 0)),
            (datetime(2024, 1, 1, 1, 0, 0), datetime(2024, 1, 1, 2, 0, 0)),
            (datetime(2024, 1, 1, 2, 0, 0), datetime(2024, 1, 1, 3, 0, 0)),
        ]
        self.assertEqual(split_ranges, expected_ranges)

    def test_split_range_single_split(self):
        conf.set("DUCKDB_RANGE_SPLIT_COUNT", 1)
        range_start = datetime(2024, 1, 1, 0, 0, 0)
        range_end = datetime(2024, 2, 1, 0, 0, 0)

        split_ranges = split_range(range_start, range_end, timedelta(days=1))

        expected_ranges = [(range_start, range_end)]
        self.assertEqual(split_ranges, expected_ranges)



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Mapping, Optional, Union

BATCH_MATERIALIZATION: JobType
BILLABLE_USAGE_OPTIONS_FIELD_NUMBER: ClassVar[int]
COMPUTE_TYPE_UNSPECIFIED: ComputeType
CONSUMPTION_TYPE_UNSPECIFIED: ConsumptionType
CONSUMPTION_UNITS_UNSPECIFIED: ConsumptionUnit
DATABRICKS: ComputeType
DATASET_GENERATION: JobType
DESCRIPTOR: _descriptor.FileDescriptor
EMR: ComputeType
ENTITY_DELETION: JobType
FEATURE_PUBLISH: JobType
FEATURE_SERVER_NODE_DURATION: ConsumptionType
FEATURE_SERVER_NODE_HOURS: ConsumptionUnit
FEATURE_SERVER_READS: ConsumptionType
FEATURE_SERVICE_ONLINE_REQUESTS: ConsumptionUnit
FEATURE_SERVICE_ONLINE_VECTORS_SERVED: ConsumptionUnit
FEATURE_SERVICE_VECTORS_SERVED: ConsumptionUnit
FEATURE_TABLE_INGEST: JobType
FEATURE_VIEW_ONLINE_READS: ConsumptionUnit
INGEST_API_COMPUTE: ConsumptionType
JOB_TYPE_UNSPECIFIED: JobType
MATERIALIZATION_JOB_WRITES: ConsumptionType
OFFLINE_WRITE_ROWS: ConsumptionUnit
OFFLINE_WRITE_VALUES: ConsumptionUnit
ONLINE_WRITE_ROWS: ConsumptionUnit
REAL_TIME_COMPUTE_DURATION_HOURS: ConsumptionUnit
REAL_TIME_JOB_COMPUTE: ConsumptionType
REQUIREMENT_NOT_REQUIRED: Requirement
REQUIREMENT_REQUIRED: Requirement
REQUIREMENT_UNSPECIFIED: Requirement
RIFT: ComputeType
RIFT_MATERIALIZATION_JOB_COMPUTE: ConsumptionType
SPARK_MATERIALIZATION_JOB_COMPUTE: ConsumptionType
STREAM_MATERIALIZATION: JobType
TECTON_JOB_COMPUTE_HOURS: ConsumptionUnit
VISIBILITY_UNSPECIFIED: Visibility
VISIBILITY_VISIBLE: Visibility
billable_usage_options: _descriptor.FieldDescriptor

class BillableUsageOptions(_message.Message):
    __slots__ = ["required", "visibility"]
    REQUIRED_FIELD_NUMBER: ClassVar[int]
    VISIBILITY_FIELD_NUMBER: ClassVar[int]
    required: Requirement
    visibility: Visibility
    def __init__(self, visibility: Optional[Union[Visibility, str]] = ..., required: Optional[Union[Requirement, str]] = ...) -> None: ...

class ConsumptionInfo(_message.Message):
    __slots__ = ["details", "feature_view_id", "feature_view_name", "metric", "online_read_aws_region", "source_id", "time_bucket_start", "units_consumed", "workspace"]
    class DetailsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    DETAILS_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    METRIC_FIELD_NUMBER: ClassVar[int]
    ONLINE_READ_AWS_REGION_FIELD_NUMBER: ClassVar[int]
    SOURCE_ID_FIELD_NUMBER: ClassVar[int]
    TIME_BUCKET_START_FIELD_NUMBER: ClassVar[int]
    UNITS_CONSUMED_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    details: _containers.ScalarMap[str, str]
    feature_view_id: _id__client_pb2.Id
    feature_view_name: str
    metric: str
    online_read_aws_region: str
    source_id: str
    time_bucket_start: _timestamp_pb2.Timestamp
    units_consumed: int
    workspace: str
    def __init__(self, time_bucket_start: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., units_consumed: Optional[int] = ..., metric: Optional[str] = ..., details: Optional[Mapping[str, str]] = ..., source_id: Optional[str] = ..., feature_view_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., feature_view_name: Optional[str] = ..., workspace: Optional[str] = ..., online_read_aws_region: Optional[str] = ...) -> None: ...

class ConsumptionRecord(_message.Message):
    __slots__ = ["account_name", "collection_timestamp", "duration", "feature_server_node_hours_metadata", "feature_server_reads_metadata", "ingest_api_compute_hours_metadata", "materialization_job_offline_writes_metadata", "materialization_job_online_writes_metadata", "quantity", "real_time_compute_metadata", "tecton_job_compute_hours_metadata", "timestamp", "unit"]
    ACCOUNT_NAME_FIELD_NUMBER: ClassVar[int]
    COLLECTION_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    DURATION_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVER_NODE_HOURS_METADATA_FIELD_NUMBER: ClassVar[int]
    FEATURE_SERVER_READS_METADATA_FIELD_NUMBER: ClassVar[int]
    INGEST_API_COMPUTE_HOURS_METADATA_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_JOB_OFFLINE_WRITES_METADATA_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_JOB_ONLINE_WRITES_METADATA_FIELD_NUMBER: ClassVar[int]
    QUANTITY_FIELD_NUMBER: ClassVar[int]
    REAL_TIME_COMPUTE_METADATA_FIELD_NUMBER: ClassVar[int]
    TECTON_JOB_COMPUTE_HOURS_METADATA_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    UNIT_FIELD_NUMBER: ClassVar[int]
    account_name: str
    collection_timestamp: _timestamp_pb2.Timestamp
    duration: _duration_pb2.Duration
    feature_server_node_hours_metadata: FeatureServerNodeHoursMetadata
    feature_server_reads_metadata: FeatureServerReadsMetadata
    ingest_api_compute_hours_metadata: IngestApiComputeHoursMetadata
    materialization_job_offline_writes_metadata: MaterializationJobOfflineWritesMetadata
    materialization_job_online_writes_metadata: MaterializationJobOnlineWritesMetadata
    quantity: float
    real_time_compute_metadata: RealTimeComputeMetadata
    tecton_job_compute_hours_metadata: TectonJobComputeHoursMetadata
    timestamp: _timestamp_pb2.Timestamp
    unit: ConsumptionUnit
    def __init__(self, timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., collection_timestamp: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., duration: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., account_name: Optional[str] = ..., materialization_job_online_writes_metadata: Optional[Union[MaterializationJobOnlineWritesMetadata, Mapping]] = ..., materialization_job_offline_writes_metadata: Optional[Union[MaterializationJobOfflineWritesMetadata, Mapping]] = ..., feature_server_node_hours_metadata: Optional[Union[FeatureServerNodeHoursMetadata, Mapping]] = ..., feature_server_reads_metadata: Optional[Union[FeatureServerReadsMetadata, Mapping]] = ..., real_time_compute_metadata: Optional[Union[RealTimeComputeMetadata, Mapping]] = ..., tecton_job_compute_hours_metadata: Optional[Union[TectonJobComputeHoursMetadata, Mapping]] = ..., ingest_api_compute_hours_metadata: Optional[Union[IngestApiComputeHoursMetadata, Mapping]] = ..., quantity: Optional[float] = ..., unit: Optional[Union[ConsumptionUnit, str]] = ...) -> None: ...

class EnrichedConsumptionInfo(_message.Message):
    __slots__ = ["consumption_info", "feature_view_id", "feature_view_name", "feature_view_workspace"]
    CONSUMPTION_INFO_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_WORKSPACE_FIELD_NUMBER: ClassVar[int]
    consumption_info: ConsumptionInfo
    feature_view_id: str
    feature_view_name: str
    feature_view_workspace: str
    def __init__(self, consumption_info: Optional[Union[ConsumptionInfo, Mapping]] = ..., feature_view_workspace: Optional[str] = ..., feature_view_name: Optional[str] = ..., feature_view_id: Optional[str] = ...) -> None: ...

class FeatureServerNodeHoursMetadata(_message.Message):
    __slots__ = ["pod_count", "pod_cpu", "pod_memory_mib", "region"]
    POD_COUNT_FIELD_NUMBER: ClassVar[int]
    POD_CPU_FIELD_NUMBER: ClassVar[int]
    POD_MEMORY_MIB_FIELD_NUMBER: ClassVar[int]
    REGION_FIELD_NUMBER: ClassVar[int]
    pod_count: int
    pod_cpu: float
    pod_memory_mib: int
    region: str
    def __init__(self, region: Optional[str] = ..., pod_cpu: Optional[float] = ..., pod_memory_mib: Optional[int] = ..., pod_count: Optional[int] = ...) -> None: ...

class FeatureServerReadsMetadata(_message.Message):
    __slots__ = ["tecton_object_id", "tecton_object_name", "workspace"]
    TECTON_OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    tecton_object_id: str
    tecton_object_name: str
    workspace: str
    def __init__(self, workspace: Optional[str] = ..., tecton_object_name: Optional[str] = ..., tecton_object_id: Optional[str] = ...) -> None: ...

class IngestApiComputeHoursMetadata(_message.Message):
    __slots__ = ["compute_type", "job_type", "memory_mib", "operation", "tags", "tecton_object_id", "tecton_object_name", "workspace", "workspace_state_id"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    COMPUTE_TYPE_FIELD_NUMBER: ClassVar[int]
    JOB_TYPE_FIELD_NUMBER: ClassVar[int]
    MEMORY_MIB_FIELD_NUMBER: ClassVar[int]
    OPERATION_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    compute_type: ComputeType
    job_type: JobType
    memory_mib: int
    operation: str
    tags: _containers.ScalarMap[str, str]
    tecton_object_id: str
    tecton_object_name: str
    workspace: str
    workspace_state_id: str
    def __init__(self, compute_type: Optional[Union[ComputeType, str]] = ..., job_type: Optional[Union[JobType, str]] = ..., operation: Optional[str] = ..., memory_mib: Optional[int] = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[str] = ..., tecton_object_name: Optional[str] = ..., tecton_object_id: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class MaterializationJobOfflineWritesMetadata(_message.Message):
    __slots__ = ["tags", "tecton_job_id", "tecton_object_id", "tecton_object_name", "workspace", "workspace_state_id"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    TAGS_FIELD_NUMBER: ClassVar[int]
    TECTON_JOB_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    tags: _containers.ScalarMap[str, str]
    tecton_job_id: str
    tecton_object_id: str
    tecton_object_name: str
    workspace: str
    workspace_state_id: str
    def __init__(self, tecton_job_id: Optional[str] = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[str] = ..., tecton_object_name: Optional[str] = ..., tecton_object_id: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class MaterializationJobOnlineWritesMetadata(_message.Message):
    __slots__ = ["online_store_type", "tags", "tecton_job_id", "tecton_object_id", "tecton_object_name", "workspace", "workspace_state_id"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    ONLINE_STORE_TYPE_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    TECTON_JOB_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    online_store_type: str
    tags: _containers.ScalarMap[str, str]
    tecton_job_id: str
    tecton_object_id: str
    tecton_object_name: str
    workspace: str
    workspace_state_id: str
    def __init__(self, online_store_type: Optional[str] = ..., tecton_job_id: Optional[str] = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[str] = ..., tecton_object_name: Optional[str] = ..., tecton_object_id: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class RealTimeComputeMetadata(_message.Message):
    __slots__ = ["memory_allocated_mib", "tags", "tecton_object_id", "tecton_object_name", "workspace", "workspace_state_id"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    MEMORY_ALLOCATED_MIB_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    memory_allocated_mib: int
    tags: _containers.ScalarMap[str, str]
    tecton_object_id: str
    tecton_object_name: str
    workspace: str
    workspace_state_id: str
    def __init__(self, memory_allocated_mib: Optional[int] = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[str] = ..., tecton_object_name: Optional[str] = ..., tecton_object_id: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class TectonJobComputeHoursMetadata(_message.Message):
    __slots__ = ["compute_type", "instance_type", "job_type", "num_workers", "region", "tags", "tecton_job_id", "tecton_object_id", "tecton_object_name", "workspace", "workspace_state_id"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    COMPUTE_TYPE_FIELD_NUMBER: ClassVar[int]
    INSTANCE_TYPE_FIELD_NUMBER: ClassVar[int]
    JOB_TYPE_FIELD_NUMBER: ClassVar[int]
    NUM_WORKERS_FIELD_NUMBER: ClassVar[int]
    REGION_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    TECTON_JOB_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_ID_FIELD_NUMBER: ClassVar[int]
    TECTON_OBJECT_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    compute_type: ComputeType
    instance_type: str
    job_type: JobType
    num_workers: int
    region: str
    tags: _containers.ScalarMap[str, str]
    tecton_job_id: str
    tecton_object_id: str
    tecton_object_name: str
    workspace: str
    workspace_state_id: str
    def __init__(self, tecton_job_id: Optional[str] = ..., instance_type: Optional[str] = ..., region: Optional[str] = ..., num_workers: Optional[int] = ..., compute_type: Optional[Union[ComputeType, str]] = ..., job_type: Optional[Union[JobType, str]] = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[str] = ..., tecton_object_name: Optional[str] = ..., tecton_object_id: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ...) -> None: ...

class JobType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class ComputeType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class ConsumptionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class ConsumptionUnit(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class Visibility(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class Requirement(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



import datetime
from dataclasses import dataclass
from typing import List
from typing import Optional
from typing import Set
from typing import Tuple
from typing import Union

from tecton import tecton_context
from tecton.aggregation_functions import AggregationFunction
from tecton.cli.error_utils import format_validation_location_fancy
from tecton.cli.printer import safe_print
from tecton.framework import base_tecton_object
from tecton.framework.base_tecton_object import BaseTectonObject
from tecton.framework.configs import LifetimeWindow
from tecton.framework.configs import TimeWindow
from tecton.framework.configs import TimeWindowSeries
from tecton.framework.configs import build_aggregation_default_name
from tecton.framework.feature_view import AggregationLeadingEdge
from tecton.framework.feature_view import FeatureView
from tecton.framework.feature_view import MaterializedFeatureView
from tecton.framework.workspace import get_workspace
from tecton.types import Array
from tecton.types import Field
from tecton.types import Map
from tecton.types import SdkDataType
from tecton.types import Struct
from tecton.v09_compat.framework import BatchFeatureView
from tecton.v09_compat.framework import DataSource
from tecton.v09_compat.framework import Entity
from tecton.v09_compat.framework import FeatureTable
from tecton.v09_compat.framework import OnDemandFeatureView
from tecton.v09_compat.framework import PushSource
from tecton.v09_compat.framework import StreamFeatureView
from tecton_core.data_types import ArrayType
from tecton_core.data_types import DataType
from tecton_core.data_types import MapType
from tecton_core.data_types import StructField
from tecton_core.data_types import StructType
from tecton_core.specs.utils import get_field_or_none
from tecton_core.time_utils import proto_to_duration
from tecton_proto.args.pipeline__client_pb2 import PipelineNode
from tecton_proto.common.aggregation_function__client_pb2 import AggregationFunctionParams


def _get_imports_for_type(type: Union[SdkDataType, Field, DataType]) -> Set[str]:
    if isinstance(type, (Array, ArrayType)):
        element_types = _get_imports_for_type(type.element_type)
        element_types.add("Array")
        return element_types
    elif isinstance(type, (Struct, StructType)):
        types = set()
        for field in type.fields:
            element_types = _get_imports_for_type(field)
            for type in element_types:
                types.add(type)
        types.add("Struct")
        return types
    elif isinstance(type, (Map, MapType)):
        key_types = _get_imports_for_type(type.key_type)
        value_types = _get_imports_for_type(type.value_type)
        return key_types | value_types | {"Map"}
    elif isinstance(type, Field):
        types = _get_imports_for_type(type.dtype)
        types.add("Field")
        return types
    elif isinstance(type, StructField):
        types = _get_imports_for_type(type.data_type)
        types.add("Field")
        return types
    else:
        return {type.__repr__()}


def timedelta_to_string(td):
    if td < datetime.timedelta(0):
        return f"-{timedelta_to_string(-td)}"

    components = []
    days = td.days
    seconds = td.seconds
    microseconds = td.microseconds

    if days:
        components.append(f"days={days}")
    if seconds:
        hours, remainder = divmod(seconds, 3600)
        minutes, seconds = divmod(remainder, 60)
        if hours:
            components.append(f"hours={hours}")
        if minutes:
            components.append(f"minutes={minutes}")
        if seconds:
            components.append(f"seconds={seconds}")
    if microseconds:
        components.append(f"microseconds={microseconds}")

    if not components:
        return "timedelta()"

    return f"timedelta({', '.join(components)})"


def _aggregation_function_to_string_render(
    aggregation_function: AggregationFunction, params: Optional[AggregationFunctionParams]
) -> str:
    if aggregation_function.base_name == "lastn":
        return f"last_distinct({params.last_n.n})"
    elif aggregation_function.base_name == "last_non_distinct_n":
        return f"last({params.last_n.n})"
    elif aggregation_function.base_name == "first_distinct_n":
        return f"first_distinct({params.first_n.n})"
    elif aggregation_function.base_name == "first_non_distinct_n":
        return f"first({params.first_n.n})"
    elif aggregation_function.base_name == "approx_count_distinct":
        return f"approx_count_distinct({params.approx_count_distinct.precision})"
    elif aggregation_function.base_name == "approx_percentile":
        return f"approx_percentile{params.approx_percentile.percentile, params.approx_percentile.precision}"
    else:
        # All other functions without explicit AggregationFunction class-overrides (ex: "mean") are passed as strings; thus the double quotes
        return f'"{aggregation_function.base_name}"'


def _get_feature_view(feature_view_name: str) -> FeatureView:
    ws = get_workspace(tecton_context.get_current_workspace())
    return ws.get_feature_view(feature_view_name)


def _get_feature_table(feature_table_name: str) -> FeatureTable:
    ws = get_workspace(tecton_context.get_current_workspace())
    return ws.get_feature_table(feature_table_name)


def _get_complete_status_emoji(is_complete: bool) -> str:
    return "✅" if is_complete else "🚫"


@dataclass
class ImportGuidance:
    tecton_import: str
    tecton_type_import: str

    def to_pretty_string(self):
        import_msg = "💡If needed, add imports:\n"
        if self.tecton_import and not self.tecton_type_import:
            return f"{import_msg}{self.tecton_import}\n"
        elif self.tecton_type_import and not self.tecton_import:
            return f"{import_msg}{self.tecton_type_import}\n"
        elif self.tecton_type_import and self.tecton_import:
            return f"{import_msg}{self.tecton_import}\n{self.tecton_type_import}\n"
        else:
            return ""


class BaseGuidance:
    _object: BaseTectonObject
    _obj_type: str
    _repo_root: str

    _tecton_imports: Set[str]
    _tecton_type_imports: Set[DataType]

    def __init__(self, _object: BaseTectonObject, _repo_root: str):
        self._object = _object
        self._repo_root = _repo_root
        self._tecton_imports = set()
        self._tecton_type_imports = set()

    def _get_upgrade_guidance(self) -> List[str]:
        raise NotImplementedError

    def _build_import_guidance(
        self,
    ) -> ImportGuidance:
        tecton_import_str = ""
        if bool(self._tecton_imports):
            tecton_import_str = f'from tecton import {", ".join(self._tecton_imports)}'

        tecton_type_import_str = ""
        if bool(self._tecton_type_imports):
            tecton_type_import_str_list = ", ".join(
                [str(tecton_type_import) for tecton_type_import in self._tecton_type_imports]
            )
            tecton_type_import_str = f"from tecton.types import {tecton_type_import_str_list}"

        return ImportGuidance(tecton_import=tecton_import_str, tecton_type_import=tecton_type_import_str)

    def _append_final_guidance(self, current_guidance: List[str]):
        if current_guidance:
            current_guidance.append("Update import from `tecton.v09_compat` to `tecton`.")
        else:
            current_guidance.append("No code change needed - just update import from `tecton.v09_compat` to `tecton`.")

    def print_guidance(self) -> None:
        format_validation_location_fancy(self._object, self._repo_root)
        safe_print("\n".join(self._get_upgrade_guidance()))
        safe_print("\n")


class DataSourceGuidance(BaseGuidance):
    _object: DataSource
    _obj_type = "DataSource"

    def _get_upgrade_guidance(self) -> List[str]:
        guidance = []
        if isinstance(self._object, PushSource):
            guidance.append(
                f"{_get_complete_status_emoji(False)} PushSource was deprecated in 0.9. Replace `PushSource` with `StreamSource` and set `stream_config=PushConfig()`. \nSee https://docs.tecton.ai/docs/release-notes/upgrade-process/to-09-upgrade-guide#step-by-step-upgrade-flow (Replace PushSource objects with a StreamSource that uses a PushConfig) for an example."
            )
        self._append_final_guidance(guidance)
        return guidance


class EntityGuidance(BaseGuidance):
    _object: Entity
    _obj_type = "Entity"

    def _find_feature_view_with_entity(self):
        for local_object in base_tecton_object._LOCAL_TECTON_OBJECTS:
            if issubclass(local_object.__class__, MaterializedFeatureView):
                fv_entities: List[Entity] = local_object.entities
                for entity in fv_entities:
                    if entity.name == self._object.name:
                        return local_object
        return None

    def _get_upgrade_guidance(self) -> List[str]:
        feature_view_with_entity = self._find_feature_view_with_entity()

        output_strs = []
        if feature_view_with_entity is None:
            output_strs.append(
                f"Unable to determine join key types of {self._object.name}. Please replace your entity with the below where <TYPE> is the type of your join key."
            )
            converted_join_keys = ", ".join(
                [f'Field("{join_key_name}", <TYPE>)' for join_key_name in self._object._spec.join_key_names]
            )
            self._tecton_type_imports.add("Field")
        else:
            feature_view = _get_feature_view(feature_view_with_entity.name)
            view_schema = feature_view._feature_definition.view_schema.to_dict()
            converted_join_keys = ", ".join(
                [
                    f'Field("{join_key_name}", {view_schema[join_key_name]})'
                    for join_key_name in self._object._spec.join_key_names
                ]
            )
            for tecton_type_import in ["Field"] + [
                view_schema[join_key_name] for join_key_name in self._object._spec.join_key_names
            ]:
                self._tecton_type_imports.add(tecton_type_import)

        output_strs.append(
            f"{_get_complete_status_emoji(False)} Replace `join_keys=[...]` with `join_keys=[{converted_join_keys}]`."
        )
        output_strs.append(self._build_import_guidance().to_pretty_string())
        self._append_final_guidance(output_strs)
        return output_strs


class MaterializedFeatureViewGuidance(BaseGuidance):
    _object: Union[BatchFeatureView, StreamFeatureView]
    _obj_type: str
    _stored_feature_view: FeatureView

    def __init__(self, _object, _repo_root):
        super().__init__(_object, _repo_root)
        self._stored_feature_view = _get_feature_view(self._object.name)
        self._view_schema = self._stored_feature_view._feature_definition.view_schema.to_dict()

    def _build_timestamp_recommendation(self):
        timestamp_field = self._stored_feature_view._feature_definition.timestamp_key
        return f'timestamp_field="{timestamp_field}",'

    def __render_time_window(self, time_window: Union[TimeWindow, TimeWindowSeries, LifetimeWindow]) -> str:
        if isinstance(time_window, TimeWindow):
            if time_window.offset == datetime.timedelta(seconds=0):
                return f"{timedelta_to_string(time_window.window_size)}"
            return (
                f"TimeWindow(window_size={timedelta_to_string(time_window.window_size)}, offset={time_window.offset})"
            )
        return time_window._to_spec().to_string()

    def _build_aggregation_replacement(self) -> List[str]:
        aggregates = []
        aggregate_name_to_function_params = {
            aggregation.output_feature_name: get_field_or_none(aggregation, "function_params")
            for aggregation in self._object._spec.aggregate_features
        }

        is_continuous = self._stored_feature_view._feature_definition.is_continuous
        aggregation_interval = self._object.aggregation_interval
        compaction_enabled = self._stored_feature_view._feature_definition.compaction_enabled

        for aggregation in self._object.aggregations:
            input_column_name = aggregation.column
            input_column_type = self._view_schema.get(input_column_name)
            aggregation_default_name = build_aggregation_default_name(
                aggregation.column,
                aggregation.time_window,
                aggregation.function,
                aggregation_interval,
                is_continuous,
                compaction_enabled,
            )

            # only set name argument if the aggregation name is not the default
            # if the aggregation name is the default, implies the user did not set the aggregation name
            name_argument = f'name="{aggregation.name}", ' if aggregation.name != aggregation_default_name else ""

            if input_column_type:
                type_imports = _get_imports_for_type(input_column_type)
                for type_import in type_imports:
                    self._tecton_type_imports.add(type_import)
            print_input_column_type = input_column_type or "<COLUMN_TYPE>"
            function_render = _aggregation_function_to_string_render(
                aggregation.function, aggregate_name_to_function_params.get(aggregation.name)
            )
            time_window = self.__render_time_window(aggregation.time_window)
            aggregate_replacement = f'Aggregate({name_argument}input_column=Field("{input_column_name}", {print_input_column_type}), function={function_render}, time_window={time_window})'
            aggregates.append(aggregate_replacement)
        return aggregates

    # Embeddings and Inference were not live in 0.9, so only need to handle attributes
    def _build_attribute_replacement(self) -> List[str]:
        attributes = []
        for attribute in self._stored_feature_view._feature_definition.features:
            dtype = self._view_schema.get(attribute)
            if dtype:
                type_imports = _get_imports_for_type(dtype)
                for type_import in type_imports:
                    self._tecton_type_imports.add(type_import)
            print_attribute_dtype = dtype or "<COLUMN_TYPE>"
            attribute = f'Attribute("{attribute}", {print_attribute_dtype})'
            attributes.append(attribute)
        return attributes

    def _get_upgrade_guidance(self) -> List[str]:
        output_steps = []
        is_timestamp_field_set = self._object._args.materialized_feature_view_args.timestamp_field != ""

        if bool(self._object.aggregations):
            output_steps.append(f"{_get_complete_status_emoji(False)} Remove `aggregations=[...]`")
            aggregate_feature_suggestions = self._build_aggregation_replacement()
            feature_string = ",\n\t".join(aggregate_feature_suggestions)
            self._tecton_imports.add("Aggregate")
            self._tecton_type_imports.add("Field")
        else:
            attribute_feature_suggestions = self._build_attribute_replacement()
            feature_string = ",\n\t".join(attribute_feature_suggestions)
            self._tecton_imports.add("Attribute")

        is_schema_removed = not self._object._args.materialized_feature_view_args.schema.columns
        output_steps.append((f"{_get_complete_status_emoji(is_schema_removed)} Remove `schema=[...]`"))
        feature_view_arguments = []
        if not is_timestamp_field_set:
            feature_view_arguments.append(self._build_timestamp_recommendation())
        feature_view_arguments.append(f"features=[\n\t{feature_string}\n]")

        is_features_set = self._object._use_feature_param()
        feature_view_arguments_str = "\n".join(feature_view_arguments)
        output_steps.append(
            f"{_get_complete_status_emoji(is_features_set)} Add feature view arguments:\n```\n{feature_view_arguments_str}\n```."
        )
        return output_steps

    def _get_data_source_nodes(self, node: PipelineNode, data_source_nodes: List[PipelineNode]):
        if node.HasField("data_source_node"):
            if node not in data_source_nodes:
                data_source_nodes.append(node.data_source_node)
            return data_source_nodes

        if node.HasField("transformation_node"):
            for input in node.transformation_node.inputs:
                self._get_data_source_nodes(input.node, data_source_nodes)

        return data_source_nodes

    def _build_filtered_source_replacements(self) -> Tuple[List[str], List[str]]:
        filtered_source_replacements = []
        args_data_source_nodes = self._get_data_source_nodes(self._object._args.pipeline.root, [])

        data_proto_data_source_nodes = self._get_data_source_nodes(
            self._stored_feature_view._feature_definition.pipeline.root, []
        )
        data_proto_data_source_nodes_name_to_node = {node.input_name: node for node in data_proto_data_source_nodes}

        currently_existing = []
        for data_source_node in args_data_source_nodes:
            existing_data_source_node = data_proto_data_source_nodes_name_to_node.get(data_source_node.input_name)
            assert existing_data_source_node, f"Cannot find data source node named {data_source_node.input_name}"

            if proto_to_duration(existing_data_source_node.start_time_offset).as_timedelta() != datetime.timedelta(0):
                start_time_offset_as_duration = proto_to_duration(
                    existing_data_source_node.start_time_offset
                ).as_timedelta()
                start_time_offset_as_string = f"{'+ ' if start_time_offset_as_duration > datetime.timedelta(0) else ''}{timedelta_to_string(start_time_offset_as_duration)}"
                currently_existing.append(
                    f"FilteredSource({data_source_node.input_name}, start_time_offset={start_time_offset_as_string})"
                )
                filtered_source_replacements.append(
                    f"{data_source_node.input_name}.select_range(\n\tstart_time = TectonTimeConstant.MATERIALIZATION_START_TIME{start_time_offset_as_string},\n\tend_time = TectonTimeConstant.MATERIALIZATION_END_TIME\n)"
                )
                self._tecton_imports.add("TectonTimeConstant")
            elif existing_data_source_node.window_unbounded:
                currently_existing.append(data_source_node.input_name)
                filtered_source_replacements.append(f"{data_source_node.input_name}.unfiltered()")
            elif existing_data_source_node.window_unbounded_preceding:
                # we do not store the offset time delta on the data source node if it >= MIN_START_OFFSET (100 years), so just stub it for currently existing.
                currently_existing.append(
                    f"FilteredSource({data_source_node.input_name}, start_time_offset=timedelta())"
                )
                filtered_source_replacements.append(
                    f"{data_source_node.input_name}.select_range(\n\tstart_time = TectonTimeConstant.UNBOUNDED_PAST,\n\tend_time = TectonTimeConstant.MATERIALIZATION_END_TIME\n)"
                )
                self._tecton_imports.add("TectonTimeConstant")
            else:
                currently_existing.append(f"FilteredSource({data_source_node.input_name})")
                filtered_source_replacements.append(f"{data_source_node.input_name}")
        return currently_existing, filtered_source_replacements


class BatchFeatureViewGuidance(MaterializedFeatureViewGuidance):
    _object: BatchFeatureView
    _obj_type = "BatchFeatureView"

    def _build_filtered_source_guidance(self) -> str:
        currently_existing, filtered_source_replacements = self._build_filtered_source_replacements()
        currently_existing_str = ", ".join(currently_existing)
        filtered_source_replacement_str = ", ".join(filtered_source_replacements)

        return f"{_get_complete_status_emoji(False)} Replace data sources:\n```\nsources=[{currently_existing_str}]\n```\nwith: \n```\nsources=[{filtered_source_replacement_str}]\n```."

    def _get_upgrade_guidance(self) -> List[str]:
        guidance = super()._get_upgrade_guidance()
        guidance.append(self._build_filtered_source_guidance())

        guidance.append(self._build_import_guidance().to_pretty_string())

        self._append_final_guidance(guidance)
        return guidance


class StreamFeatureViewGuidance(MaterializedFeatureViewGuidance):
    _object: StreamFeatureView
    _obj_type = "StreamFeatureView"

    def _build_filtered_source_guidance(self) -> str:
        currently_existing, filtered_source_replacements = self._build_filtered_source_replacements()
        currently_existing_str = ", ".join(currently_existing)
        filtered_source_replacement_str = ", ".join(filtered_source_replacements)
        return f"{_get_complete_status_emoji(False)} Replace `source={currently_existing_str}` with `source={filtered_source_replacement_str}`."

    def _get_aggregation_leading_edge_guidance(self) -> str:
        is_aggregation_leading_edge_set = (
            self._object._args.materialized_feature_view_args.aggregation_leading_edge
            == AggregationLeadingEdge.UNSPECIFIED
        )
        if not is_aggregation_leading_edge_set:
            self._tecton_imports.add("AggregationLeadingEdge")
        return f"{_get_complete_status_emoji(is_aggregation_leading_edge_set)} Set `aggregation_leading_edge=AggregationLeadingEdge.LATEST_EVENT_TIME`."

    def _get_upgrade_guidance(self) -> List[str]:
        guidance = super()._get_upgrade_guidance()
        guidance.append(self._build_filtered_source_guidance())
        guidance.append(self._get_aggregation_leading_edge_guidance())

        guidance.append(self._build_import_guidance().to_pretty_string())

        self._append_final_guidance(guidance)
        return guidance


class OnDemandFeatureViewGuidance(BaseGuidance):
    _object: OnDemandFeatureView
    _obj_type = "OnDemandFeatureView"

    def __init__(self, _object, _repo_root):
        super().__init__(_object, _repo_root)
        self._stored_feature_view = _get_feature_view(self._object.name)
        self._view_schema = self._stored_feature_view._feature_definition.view_schema.to_dict()

    # Embeddings and Inference were not live in 0.9, so only need to handle attributes
    def _build_attribute_replacement(self) -> List[str]:
        self._tecton_imports.add("Attribute")
        attributes = []
        for attribute in self._stored_feature_view._feature_definition.features:
            attribute_dtype = self._view_schema.get(attribute)
            if attribute_dtype:
                type_imports = _get_imports_for_type(attribute_dtype)
                for type_import in type_imports:
                    self._tecton_type_imports.add(type_import)
            attribute_dtype_str = attribute_dtype or "<COLUMN_TYPE>"
            attribute = f'Attribute("{attribute}", {attribute_dtype_str})'
            attributes.append(attribute)
        return attributes

    def _get_upgrade_guidance(self) -> List[str]:
        output_steps = []
        is_schema_removed = not self._object._args.realtime_args.schema.fields
        output_steps.append((f"{_get_complete_status_emoji(is_schema_removed)} Remove `schema=[...]`"))

        is_features_set = self._object._use_feature_param()
        attribute_feature_suggestions = self._build_attribute_replacement()
        attribute_feature_string = ",\n\t".join(attribute_feature_suggestions)
        output_steps.append(
            f"{_get_complete_status_emoji(is_features_set)} Add \n```\nfeatures=[\n\t{attribute_feature_string}\n]\n```."
        )

        output_steps.append(self._build_import_guidance().to_pretty_string())

        output_steps.append(
            f"{_get_complete_status_emoji(False)} Rename from `on_demand_feature_view` to realtime_feature_view` and update import from `tecton.v09_compat.on_demand_feature_view` to `tecton.realtime_feature_view`."
        )
        return output_steps


class FeatureTableGuidance(BaseGuidance):
    _object: FeatureTable
    _obj_type = "FeatureTable"

    def __init__(self, _object, _repo_root):
        super().__init__(_object, _repo_root)
        self._stored_feature_view = _get_feature_table(self._object.name)
        self._view_schema = self._stored_feature_view._feature_definition.view_schema.to_dict()

    def _build_timestamp_recommendation(self):
        timestamp_field = self._stored_feature_view._feature_definition.timestamp_key
        return f'Set `timestamp_field="{timestamp_field}"`.'

    # Embeddings and Inference were not live in 0.9, so only need to handle attributes
    def _build_attribute_replacement(self) -> List[str]:
        self._tecton_imports.add("Attribute")
        attributes = []
        for attribute in self._stored_feature_view._feature_definition.features:
            attribute_dtype = self._view_schema.get(attribute)
            if attribute_dtype:
                type_imports = _get_imports_for_type(attribute_dtype)
                for type_import in type_imports:
                    self._tecton_type_imports.add(type_import)
            attribute_dtype_str = attribute_dtype or "<COLUMN_TYPE>"
            attribute = f'Attribute("{attribute}", {attribute_dtype_str})'
            attributes.append(attribute)
        return attributes

    def _get_upgrade_guidance(self) -> List[str]:
        output_steps = []
        is_timestamp_field_set = self._object._args.feature_table_args.timestamp_field != ""
        output_steps.append(
            f"{_get_complete_status_emoji(is_timestamp_field_set)} {self._build_timestamp_recommendation()}"
        )

        is_schema_removed = not self._object._args.feature_table_args.schema.fields
        output_steps.append((f"{_get_complete_status_emoji(is_schema_removed)} Remove `schema=[...]`"))

        is_features_set = self._object._use_feature_param()
        attribute_feature_suggestions = self._build_attribute_replacement()
        attribute_feature_string = ",\n\t".join(attribute_feature_suggestions)
        output_steps.append(
            f"{_get_complete_status_emoji(is_features_set)} Add \n```\nfeatures=[\n\t{attribute_feature_string}\n]\n```."
        )

        output_steps.append(self._build_import_guidance().to_pretty_string())

        self._append_final_guidance(output_steps)
        return output_steps



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import aws_credentials__client_pb2 as _aws_credentials__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from tecton_proto.data import feature_view__client_pb2 as _feature_view__client_pb2
from tecton_proto.data import user_deployment_settings__client_pb2 as _user_deployment_settings__client_pb2
from tecton_proto.dataobs import expectation__client_pb2 as _expectation__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class DynamoDataSource(_message.Message):
    __slots__ = ["dynamo_role", "feature_end_time", "feature_start_time", "metric_interval", "table_name"]
    DYNAMO_ROLE_FIELD_NUMBER: ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    METRIC_INTERVAL_FIELD_NUMBER: ClassVar[int]
    TABLE_NAME_FIELD_NUMBER: ClassVar[int]
    dynamo_role: _aws_credentials__client_pb2.AwsIamRole
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    metric_interval: _duration_pb2.Duration
    table_name: str
    def __init__(self, dynamo_role: Optional[Union[_aws_credentials__client_pb2.AwsIamRole, Mapping]] = ..., table_name: Optional[str] = ..., metric_interval: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...

class S3DataSource(_message.Message):
    __slots__ = ["location"]
    LOCATION_FIELD_NUMBER: ClassVar[int]
    location: str
    def __init__(self, location: Optional[str] = ...) -> None: ...

class ValidationTaskParams(_message.Message):
    __slots__ = ["dynamo_data_source", "feature_view", "metric_expectations", "result_key", "result_location", "s3_data_source", "task_id", "validation_job_id", "workspace_name"]
    DYNAMO_DATA_SOURCE_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_FIELD_NUMBER: ClassVar[int]
    METRIC_EXPECTATIONS_FIELD_NUMBER: ClassVar[int]
    RESULT_KEY_FIELD_NUMBER: ClassVar[int]
    RESULT_LOCATION_FIELD_NUMBER: ClassVar[int]
    S3_DATA_SOURCE_FIELD_NUMBER: ClassVar[int]
    TASK_ID_FIELD_NUMBER: ClassVar[int]
    VALIDATION_JOB_ID_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_NAME_FIELD_NUMBER: ClassVar[int]
    dynamo_data_source: DynamoDataSource
    feature_view: _feature_view__client_pb2.FeatureView
    metric_expectations: _containers.RepeatedCompositeFieldContainer[_expectation__client_pb2.MetricExpectation]
    result_key: str
    result_location: _user_deployment_settings__client_pb2.S3Location
    s3_data_source: S3DataSource
    task_id: int
    validation_job_id: _id__client_pb2.Id
    workspace_name: str
    def __init__(self, validation_job_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., task_id: Optional[int] = ..., workspace_name: Optional[str] = ..., feature_view: Optional[Union[_feature_view__client_pb2.FeatureView, Mapping]] = ..., metric_expectations: Optional[Iterable[Union[_expectation__client_pb2.MetricExpectation, Mapping]]] = ..., dynamo_data_source: Optional[Union[DynamoDataSource, Mapping]] = ..., s3_data_source: Optional[Union[S3DataSource, Mapping]] = ..., result_location: Optional[Union[_user_deployment_settings__client_pb2.S3Location, Mapping]] = ..., result_key: Optional[str] = ...) -> None: ...



from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.args import diff_options__client_pb2 as _diff_options__client_pb2
from tecton_proto.args import fco_args__client_pb2 as _fco_args__client_pb2
from tecton_proto.args import repo_metadata__client_pb2 as _repo_metadata__client_pb2
from tecton_proto.auth import principal__client_pb2 as _principal__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

AUTO: PlanIntegrationTestSelectType
CREATE: FcoTransitionType
DELETE: FcoTransitionType
DESCRIPTOR: _descriptor.FileDescriptor
JOB_STATUS_CANCELLED: IntegrationTestJobStatus
JOB_STATUS_FAILED: IntegrationTestJobStatus
JOB_STATUS_NOT_STARTED: IntegrationTestJobStatus
JOB_STATUS_RUNNING: IntegrationTestJobStatus
JOB_STATUS_SUCCEED: IntegrationTestJobStatus
JOB_STATUS_UNSPECIFIED: IntegrationTestJobStatus
MATERIALIZATION_TASK_DIFF_DESTINATION_BULK_LOAD_ONLINE: MaterializationTaskDiffDestination
MATERIALIZATION_TASK_DIFF_DESTINATION_OFFLINE: MaterializationTaskDiffDestination
MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE: MaterializationTaskDiffDestination
MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE_AND_OFFLINE: MaterializationTaskDiffDestination
MATERIALIZATION_TASK_DIFF_DESTINATION_UNSPECIFIED: MaterializationTaskDiffDestination
NONE: PlanIntegrationTestSelectType
PLAN_APPLIED: PlanStatusType
PLAN_APPLY_FAILED: PlanStatusType
PLAN_CREATED: PlanStatusType
PLAN_INTEGRATION_TESTS_CANCELLED: PlanStatusType
PLAN_INTEGRATION_TESTS_FAILED: PlanStatusType
PLAN_INTEGRATION_TESTS_NOT_STARTED: PlanStatusType
PLAN_INTEGRATION_TESTS_RUNNING: PlanStatusType
PLAN_INTEGRATION_TESTS_SKIPPED: PlanStatusType
PLAN_INTEGRATION_TESTS_SUCCEED: PlanStatusType
PLAN_UNSPECIFIED: PlanStatusType
RECREATE: FcoTransitionType
RESTART_STREAM_CHECKPOINTS_INVALIDATED: FcoTransitionSideEffectStreamRestartType
RESTART_STREAM_NONE: FcoTransitionSideEffectStreamRestartType
RESTART_STREAM_REUSE_CHECKPOINTS: FcoTransitionSideEffectStreamRestartType
SELECTED_FEATURE_VIEWS: PlanIntegrationTestSelectType
UNCHANGED: FcoTransitionType
UNKNOWN: FcoTransitionType
UNSPECIFIED: PlanIntegrationTestSelectType
UPDATE: FcoTransitionType
UPGRADE: FcoTransitionType

class BackfillFeaturePublishTaskDiff(_message.Message):
    __slots__ = ["display_string", "feature_end_time", "feature_start_time", "number_of_jobs"]
    DISPLAY_STRING_FIELD_NUMBER: ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    NUMBER_OF_JOBS_FIELD_NUMBER: ClassVar[int]
    display_string: str
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    number_of_jobs: int
    def __init__(self, display_string: Optional[str] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., number_of_jobs: Optional[int] = ...) -> None: ...

class BackfillMaterializationTaskDiff(_message.Message):
    __slots__ = ["destination", "display_string", "feature_end_time", "feature_start_time", "number_of_jobs"]
    DESTINATION_FIELD_NUMBER: ClassVar[int]
    DISPLAY_STRING_FIELD_NUMBER: ClassVar[int]
    FEATURE_END_TIME_FIELD_NUMBER: ClassVar[int]
    FEATURE_START_TIME_FIELD_NUMBER: ClassVar[int]
    NUMBER_OF_JOBS_FIELD_NUMBER: ClassVar[int]
    destination: MaterializationTaskDiffDestination
    display_string: str
    feature_end_time: _timestamp_pb2.Timestamp
    feature_start_time: _timestamp_pb2.Timestamp
    number_of_jobs: int
    def __init__(self, display_string: Optional[str] = ..., feature_start_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., feature_end_time: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., number_of_jobs: Optional[int] = ..., destination: Optional[Union[MaterializationTaskDiffDestination, str]] = ...) -> None: ...

class BatchMaterializationTaskDiff(_message.Message):
    __slots__ = ["destination", "display_string", "schedule_interval"]
    DESTINATION_FIELD_NUMBER: ClassVar[int]
    DISPLAY_STRING_FIELD_NUMBER: ClassVar[int]
    SCHEDULE_INTERVAL_FIELD_NUMBER: ClassVar[int]
    destination: MaterializationTaskDiffDestination
    display_string: str
    schedule_interval: _duration_pb2.Duration
    def __init__(self, display_string: Optional[str] = ..., schedule_interval: Optional[Union[_duration_pb2.Duration, Mapping]] = ..., destination: Optional[Union[MaterializationTaskDiffDestination, str]] = ...) -> None: ...

class FcoDiff(_message.Message):
    __slots__ = ["declared_args", "diff", "existing_args", "materialization_info", "transition_side_effects", "type"]
    DECLARED_ARGS_FIELD_NUMBER: ClassVar[int]
    DIFF_FIELD_NUMBER: ClassVar[int]
    EXISTING_ARGS_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_INFO_FIELD_NUMBER: ClassVar[int]
    TRANSITION_SIDE_EFFECTS_FIELD_NUMBER: ClassVar[int]
    TYPE_FIELD_NUMBER: ClassVar[int]
    declared_args: _fco_args__client_pb2.FcoArgs
    diff: _containers.RepeatedCompositeFieldContainer[FcoPropertyDiff]
    existing_args: _fco_args__client_pb2.FcoArgs
    materialization_info: MaterializationInfo
    transition_side_effects: FcoTransitionSideEffects
    type: FcoTransitionType
    def __init__(self, type: Optional[Union[FcoTransitionType, str]] = ..., transition_side_effects: Optional[Union[FcoTransitionSideEffects, Mapping]] = ..., existing_args: Optional[Union[_fco_args__client_pb2.FcoArgs, Mapping]] = ..., declared_args: Optional[Union[_fco_args__client_pb2.FcoArgs, Mapping]] = ..., diff: Optional[Iterable[Union[FcoPropertyDiff, Mapping]]] = ..., materialization_info: Optional[Union[MaterializationInfo, Mapping]] = ...) -> None: ...

class FcoFieldRef(_message.Message):
    __slots__ = ["fco_id"]
    FCO_ID_FIELD_NUMBER: ClassVar[int]
    fco_id: _id__client_pb2.Id
    def __init__(self, fco_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ...) -> None: ...

class FcoPropertyDiff(_message.Message):
    __slots__ = ["custom_comparator", "property_name", "rendering_type", "val_declared", "val_existing"]
    CUSTOM_COMPARATOR_FIELD_NUMBER: ClassVar[int]
    PROPERTY_NAME_FIELD_NUMBER: ClassVar[int]
    RENDERING_TYPE_FIELD_NUMBER: ClassVar[int]
    VAL_DECLARED_FIELD_NUMBER: ClassVar[int]
    VAL_EXISTING_FIELD_NUMBER: ClassVar[int]
    custom_comparator: _diff_options__client_pb2.CustomComparator
    property_name: str
    rendering_type: _diff_options__client_pb2.FcoPropertyRenderingType
    val_declared: str
    val_existing: str
    def __init__(self, property_name: Optional[str] = ..., val_existing: Optional[str] = ..., val_declared: Optional[str] = ..., rendering_type: Optional[Union[_diff_options__client_pb2.FcoPropertyRenderingType, str]] = ..., custom_comparator: Optional[Union[_diff_options__client_pb2.CustomComparator, str]] = ...) -> None: ...

class FcoTransitionSideEffects(_message.Message):
    __slots__ = ["stream_restart_type"]
    STREAM_RESTART_TYPE_FIELD_NUMBER: ClassVar[int]
    stream_restart_type: FcoTransitionSideEffectStreamRestartType
    def __init__(self, stream_restart_type: Optional[Union[FcoTransitionSideEffectStreamRestartType, str]] = ...) -> None: ...

class IntegrationTestJobSummary(_message.Message):
    __slots__ = ["status"]
    STATUS_FIELD_NUMBER: ClassVar[int]
    status: IntegrationTestJobStatus
    def __init__(self, status: Optional[Union[IntegrationTestJobStatus, str]] = ...) -> None: ...

class MaterializationInfo(_message.Message):
    __slots__ = ["backfill_publish_task_diffs", "backfill_task_diffs", "batch_task_diff", "integration_test_task_diffs", "stream_task_diff"]
    BACKFILL_PUBLISH_TASK_DIFFS_FIELD_NUMBER: ClassVar[int]
    BACKFILL_TASK_DIFFS_FIELD_NUMBER: ClassVar[int]
    BATCH_TASK_DIFF_FIELD_NUMBER: ClassVar[int]
    INTEGRATION_TEST_TASK_DIFFS_FIELD_NUMBER: ClassVar[int]
    STREAM_TASK_DIFF_FIELD_NUMBER: ClassVar[int]
    backfill_publish_task_diffs: _containers.RepeatedCompositeFieldContainer[BackfillFeaturePublishTaskDiff]
    backfill_task_diffs: _containers.RepeatedCompositeFieldContainer[BackfillMaterializationTaskDiff]
    batch_task_diff: BatchMaterializationTaskDiff
    integration_test_task_diffs: _containers.RepeatedCompositeFieldContainer[PlanIntegrationTestTaskDiff]
    stream_task_diff: StreamMaterializationTaskDiff
    def __init__(self, backfill_task_diffs: Optional[Iterable[Union[BackfillMaterializationTaskDiff, Mapping]]] = ..., batch_task_diff: Optional[Union[BatchMaterializationTaskDiff, Mapping]] = ..., stream_task_diff: Optional[Union[StreamMaterializationTaskDiff, Mapping]] = ..., backfill_publish_task_diffs: Optional[Iterable[Union[BackfillFeaturePublishTaskDiff, Mapping]]] = ..., integration_test_task_diffs: Optional[Iterable[Union[PlanIntegrationTestTaskDiff, Mapping]]] = ...) -> None: ...

class PlanIntegrationTestConfig(_message.Message):
    __slots__ = ["auto_apply_upon_test_success", "feature_view_names"]
    AUTO_APPLY_UPON_TEST_SUCCESS_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAMES_FIELD_NUMBER: ClassVar[int]
    auto_apply_upon_test_success: bool
    feature_view_names: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, auto_apply_upon_test_success: bool = ..., feature_view_names: Optional[Iterable[str]] = ...) -> None: ...

class PlanIntegrationTestSummary(_message.Message):
    __slots__ = ["feature_view_id", "feature_view_name", "job_summaries"]
    FEATURE_VIEW_ID_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    JOB_SUMMARIES_FIELD_NUMBER: ClassVar[int]
    feature_view_id: _id__client_pb2.Id
    feature_view_name: str
    job_summaries: _containers.RepeatedCompositeFieldContainer[IntegrationTestJobSummary]
    def __init__(self, feature_view_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., job_summaries: Optional[Iterable[Union[IntegrationTestJobSummary, Mapping]]] = ..., feature_view_name: Optional[str] = ...) -> None: ...

class PlanIntegrationTestTaskDiff(_message.Message):
    __slots__ = ["display_string", "feature_view_name"]
    DISPLAY_STRING_FIELD_NUMBER: ClassVar[int]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    display_string: str
    feature_view_name: str
    def __init__(self, display_string: Optional[str] = ..., feature_view_name: Optional[str] = ...) -> None: ...

class StateUpdateEntry(_message.Message):
    __slots__ = ["applied_at", "applied_by", "applied_by_principal", "commit_id", "created_at", "created_by", "error", "sdk_version", "status_type", "successful_plan_output", "workspace"]
    APPLIED_AT_FIELD_NUMBER: ClassVar[int]
    APPLIED_BY_FIELD_NUMBER: ClassVar[int]
    APPLIED_BY_PRINCIPAL_FIELD_NUMBER: ClassVar[int]
    COMMIT_ID_FIELD_NUMBER: ClassVar[int]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    CREATED_BY_FIELD_NUMBER: ClassVar[int]
    ERROR_FIELD_NUMBER: ClassVar[int]
    SDK_VERSION_FIELD_NUMBER: ClassVar[int]
    STATUS_TYPE_FIELD_NUMBER: ClassVar[int]
    SUCCESSFUL_PLAN_OUTPUT_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    applied_at: _timestamp_pb2.Timestamp
    applied_by: str
    applied_by_principal: _principal__client_pb2.PrincipalBasic
    commit_id: str
    created_at: _timestamp_pb2.Timestamp
    created_by: str
    error: str
    sdk_version: str
    status_type: PlanStatusType
    successful_plan_output: SuccessfulPlanOutput
    workspace: str
    def __init__(self, commit_id: Optional[str] = ..., applied_by: Optional[str] = ..., applied_by_principal: Optional[Union[_principal__client_pb2.PrincipalBasic, Mapping]] = ..., applied_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., workspace: Optional[str] = ..., sdk_version: Optional[str] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., status_type: Optional[Union[PlanStatusType, str]] = ..., error: Optional[str] = ..., successful_plan_output: Optional[Union[SuccessfulPlanOutput, Mapping]] = ..., created_by: Optional[str] = ...) -> None: ...

class StateUpdatePlanSummary(_message.Message):
    __slots__ = ["applied_at", "applied_by", "applied_by_principal", "created_at", "created_by", "diff_items", "sdk_version", "workspace"]
    APPLIED_AT_FIELD_NUMBER: ClassVar[int]
    APPLIED_BY_FIELD_NUMBER: ClassVar[int]
    APPLIED_BY_PRINCIPAL_FIELD_NUMBER: ClassVar[int]
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    CREATED_BY_FIELD_NUMBER: ClassVar[int]
    DIFF_ITEMS_FIELD_NUMBER: ClassVar[int]
    SDK_VERSION_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    applied_at: _timestamp_pb2.Timestamp
    applied_by: str
    applied_by_principal: _principal__client_pb2.PrincipalBasic
    created_at: _timestamp_pb2.Timestamp
    created_by: str
    diff_items: _containers.RepeatedCompositeFieldContainer[StateUpdatePlanSummaryDiff]
    sdk_version: str
    workspace: str
    def __init__(self, diff_items: Optional[Iterable[Union[StateUpdatePlanSummaryDiff, Mapping]]] = ..., applied_by: Optional[str] = ..., applied_by_principal: Optional[Union[_principal__client_pb2.PrincipalBasic, Mapping]] = ..., created_by: Optional[str] = ..., applied_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., workspace: Optional[str] = ..., sdk_version: Optional[str] = ...) -> None: ...

class StateUpdatePlanSummaryDiff(_message.Message):
    __slots__ = ["description", "diffs", "fco_type", "materialization_info", "name", "transition_side_effects", "type"]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    DIFFS_FIELD_NUMBER: ClassVar[int]
    FCO_TYPE_FIELD_NUMBER: ClassVar[int]
    MATERIALIZATION_INFO_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    TRANSITION_SIDE_EFFECTS_FIELD_NUMBER: ClassVar[int]
    TYPE_FIELD_NUMBER: ClassVar[int]
    description: str
    diffs: _containers.RepeatedCompositeFieldContainer[FcoPropertyDiff]
    fco_type: str
    materialization_info: MaterializationInfo
    name: str
    transition_side_effects: FcoTransitionSideEffects
    type: FcoTransitionType
    def __init__(self, fco_type: Optional[str] = ..., type: Optional[Union[FcoTransitionType, str]] = ..., transition_side_effects: Optional[Union[FcoTransitionSideEffects, Mapping]] = ..., diffs: Optional[Iterable[Union[FcoPropertyDiff, Mapping]]] = ..., name: Optional[str] = ..., description: Optional[str] = ..., materialization_info: Optional[Union[MaterializationInfo, Mapping]] = ...) -> None: ...

class StateUpdateRequest(_message.Message):
    __slots__ = ["fco_args", "plan_integration_config", "plan_integration_type", "repo_source_info", "requested_by", "requested_by_principal", "sdk_version", "suppress_recreates", "upgrade_all", "workspace"]
    FCO_ARGS_FIELD_NUMBER: ClassVar[int]
    PLAN_INTEGRATION_CONFIG_FIELD_NUMBER: ClassVar[int]
    PLAN_INTEGRATION_TYPE_FIELD_NUMBER: ClassVar[int]
    REPO_SOURCE_INFO_FIELD_NUMBER: ClassVar[int]
    REQUESTED_BY_FIELD_NUMBER: ClassVar[int]
    REQUESTED_BY_PRINCIPAL_FIELD_NUMBER: ClassVar[int]
    SDK_VERSION_FIELD_NUMBER: ClassVar[int]
    SUPPRESS_RECREATES_FIELD_NUMBER: ClassVar[int]
    UPGRADE_ALL_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    fco_args: _containers.RepeatedCompositeFieldContainer[_fco_args__client_pb2.FcoArgs]
    plan_integration_config: PlanIntegrationTestConfig
    plan_integration_type: PlanIntegrationTestSelectType
    repo_source_info: _repo_metadata__client_pb2.FeatureRepoSourceInfo
    requested_by: str
    requested_by_principal: _principal__client_pb2.Principal
    sdk_version: str
    suppress_recreates: bool
    upgrade_all: bool
    workspace: str
    def __init__(self, workspace: Optional[str] = ..., fco_args: Optional[Iterable[Union[_fco_args__client_pb2.FcoArgs, Mapping]]] = ..., repo_source_info: Optional[Union[_repo_metadata__client_pb2.FeatureRepoSourceInfo, Mapping]] = ..., suppress_recreates: bool = ..., upgrade_all: bool = ..., requested_by: Optional[str] = ..., requested_by_principal: Optional[Union[_principal__client_pb2.Principal, Mapping]] = ..., sdk_version: Optional[str] = ..., plan_integration_type: Optional[Union[PlanIntegrationTestSelectType, str]] = ..., plan_integration_config: Optional[Union[PlanIntegrationTestConfig, Mapping]] = ...) -> None: ...

class StreamMaterializationTaskDiff(_message.Message):
    __slots__ = ["display_string"]
    DISPLAY_STRING_FIELD_NUMBER: ClassVar[int]
    display_string: str
    def __init__(self, display_string: Optional[str] = ...) -> None: ...

class SuccessfulPlanOutput(_message.Message):
    __slots__ = ["apply_warnings", "json_output", "num_fcos_changed", "num_warnings", "plan_url", "string_output", "test_summaries"]
    APPLY_WARNINGS_FIELD_NUMBER: ClassVar[int]
    JSON_OUTPUT_FIELD_NUMBER: ClassVar[int]
    NUM_FCOS_CHANGED_FIELD_NUMBER: ClassVar[int]
    NUM_WARNINGS_FIELD_NUMBER: ClassVar[int]
    PLAN_URL_FIELD_NUMBER: ClassVar[int]
    STRING_OUTPUT_FIELD_NUMBER: ClassVar[int]
    TEST_SUMMARIES_FIELD_NUMBER: ClassVar[int]
    apply_warnings: str
    json_output: str
    num_fcos_changed: int
    num_warnings: int
    plan_url: str
    string_output: str
    test_summaries: _containers.RepeatedCompositeFieldContainer[PlanIntegrationTestSummary]
    def __init__(self, string_output: Optional[str] = ..., json_output: Optional[str] = ..., apply_warnings: Optional[str] = ..., num_fcos_changed: Optional[int] = ..., num_warnings: Optional[int] = ..., test_summaries: Optional[Iterable[Union[PlanIntegrationTestSummary, Mapping]]] = ..., plan_url: Optional[str] = ...) -> None: ...

class ValidationMessage(_message.Message):
    __slots__ = ["fco_refs", "message"]
    FCO_REFS_FIELD_NUMBER: ClassVar[int]
    MESSAGE_FIELD_NUMBER: ClassVar[int]
    fco_refs: _containers.RepeatedCompositeFieldContainer[FcoFieldRef]
    message: str
    def __init__(self, message: Optional[str] = ..., fco_refs: Optional[Iterable[Union[FcoFieldRef, Mapping]]] = ...) -> None: ...

class ValidationResult(_message.Message):
    __slots__ = ["errors", "warnings"]
    ERRORS_FIELD_NUMBER: ClassVar[int]
    WARNINGS_FIELD_NUMBER: ClassVar[int]
    errors: _containers.RepeatedCompositeFieldContainer[ValidationMessage]
    warnings: _containers.RepeatedCompositeFieldContainer[ValidationMessage]
    def __init__(self, errors: Optional[Iterable[Union[ValidationMessage, Mapping]]] = ..., warnings: Optional[Iterable[Union[ValidationMessage, Mapping]]] = ...) -> None: ...

class FcoTransitionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class FcoTransitionSideEffectStreamRestartType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MaterializationTaskDiffDestination(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class IntegrationTestJobStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class PlanIntegrationTestSelectType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class PlanStatusType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from tecton_proto.auth import service__client_pb2 as _service__client_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Optional, Text

DESCRIPTOR: _descriptor.FileDescriptor

class IngestionServerConfigRequest(_message.Message):
    __slots__ = ["absolute_filepath"]
    ABSOLUTE_FILEPATH_FIELD_NUMBER: ClassVar[int]
    absolute_filepath: str
    def __init__(self, absolute_filepath: Optional[str] = ...) -> None: ...

class IngestionServerConfigResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class InitializeStoreRequest(_message.Message):
    __slots__ = ["feature_view_name", "workspace"]
    FEATURE_VIEW_NAME_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    feature_view_name: str
    workspace: str
    def __init__(self, workspace: Optional[str] = ..., feature_view_name: Optional[str] = ...) -> None: ...

class InitializeStoreResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from tecton_proto.common import framework_version__client_pb2 as _framework_version__client_pb2
from tecton_proto.common import id__client_pb2 as _id__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class FcoMetadata(_message.Message):
    __slots__ = ["created_at", "description", "family", "framework_version", "is_archived", "last_modified_by", "last_updated_at", "name", "owner", "scope", "source_filename", "source_lineno", "tags", "workspace", "workspace_state_id"]
    class TagsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    CREATED_AT_FIELD_NUMBER: ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    FAMILY_FIELD_NUMBER: ClassVar[int]
    FRAMEWORK_VERSION_FIELD_NUMBER: ClassVar[int]
    IS_ARCHIVED_FIELD_NUMBER: ClassVar[int]
    LAST_MODIFIED_BY_FIELD_NUMBER: ClassVar[int]
    LAST_UPDATED_AT_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    OWNER_FIELD_NUMBER: ClassVar[int]
    SCOPE_FIELD_NUMBER: ClassVar[int]
    SOURCE_FILENAME_FIELD_NUMBER: ClassVar[int]
    SOURCE_LINENO_FIELD_NUMBER: ClassVar[int]
    TAGS_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_STATE_ID_FIELD_NUMBER: ClassVar[int]
    created_at: _timestamp_pb2.Timestamp
    description: str
    family: str
    framework_version: _framework_version__client_pb2.FrameworkVersion
    is_archived: bool
    last_modified_by: str
    last_updated_at: _timestamp_pb2.Timestamp
    name: str
    owner: str
    scope: str
    source_filename: str
    source_lineno: str
    tags: _containers.ScalarMap[str, str]
    workspace: str
    workspace_state_id: _id__client_pb2.Id
    def __init__(self, name: Optional[str] = ..., description: Optional[str] = ..., created_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ..., owner: Optional[str] = ..., last_modified_by: Optional[str] = ..., is_archived: bool = ..., workspace: Optional[str] = ..., workspace_state_id: Optional[Union[_id__client_pb2.Id, Mapping]] = ..., family: Optional[str] = ..., scope: Optional[str] = ..., source_lineno: Optional[str] = ..., source_filename: Optional[str] = ..., tags: Optional[Mapping[str, str]] = ..., framework_version: Optional[Union[_framework_version__client_pb2.FrameworkVersion, str]] = ..., last_updated_at: Optional[Union[_timestamp_pb2.Timestamp, Mapping]] = ...) -> None: ...



import glob
import os
import shutil
import tempfile
from pathlib import Path
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

import pandas
import pyarrow

from tecton._internals import model_utils
from tecton._internals import type_utils
from tecton._internals.tecton_pydantic import StrictModel
from tecton.types import Field
from tecton_core.embeddings.artifacts_provider import InferenceDeviceHardware
from tecton_core.errors import TectonValidationError
from tecton_core.schema import Column


_CUSTOM_MODELS: List["ModelConfig"] = []
_LOCAL_CUSTOM_MODELS: Dict[str, "ModelConfig"] = {}


class ModelConfig(StrictModel):
    """A Custom Model in Tecton

    ```python
    from tecton import ModelConfig

    model_config = ModelConfig(
        name=”mymodel_text_embedder”,
        model_type="pytorch",
        model_file="model_file.py",
        environments=["custom-ml-env-01"],
        artifact_files=["model.safetensors", "tokenizer_config.json" …],
        input_schema=[Field(”my_text_col”, String), …],
        output_schema=Field(”text_embedding”)
    )
    ```

    :param name: Unique name of model
    :param model_type: Type of Model (pytorch or text embedding)
    :param description: Description of Model
    :param tags: Tags associated with this Tecton Object (key-value pairs of arbitrary metadata).
    :param model_file: Path of File containing model relative to where the Tecton object is defined.
    :param artifact_files: Path of other files needed to load and run model relative to where the Tecton object is defined.
    :param environments: All the environments allowed for this custom model to run in.
    :param input_schema: Input Schema for model. Each field in this schema corresponds to the `input_columns` specified in the `Inference` and is mapped by order.
    :param output_schema: Output Schema of model.

    :raises TectonValidationError: if the input non-parameters are invalid.
    """

    name: str
    model_type: str
    description: Optional[str]
    tags: Optional[Dict[str, str]]
    model_file: str
    artifact_files: Optional[List[str]]
    environments: List[str]
    input_schema: List[Field]
    output_schema: Field

    def __init__(
        self,
        *,
        name: str,
        model_type: str,
        model_file: str,
        environments: List[str],
        input_schema: List[Field],
        output_schema: Field,
        artifact_files: Optional[List[str]] = None,
        description: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        super().__init__(
            name=name,
            model_type=model_type,
            model_file=model_file,
            artifact_files=artifact_files,
            environments=environments,
            input_schema=input_schema,
            output_schema=output_schema,
            description=description,
            tags=tags,
        )
        _CUSTOM_MODELS.append(self)

    def run(self, mock_inputs: Dict[str, List[Any]]) -> pandas.DataFrame:
        """Test Run a Model.

        :param mock_inputs: Mock Input for each columns defined in input schema
        :return: pandas.Dataframe with inputs and output columns.

        :raises TectonValidationError: if the input non-parameters are invalid.
        """
        try:
            import torch

            from tecton_core.embeddings.custom_model import CustomModelContainer
            from tecton_core.embeddings.custom_model import custom_model_inference
        except ImportError:
            msg = "Install tecton[ml-extras] to install additional packages needed to use Tecton Custom Models"
            raise ImportError(msg)

        model_file_path = Path(".") / self.model_file
        artifact_files = self.artifact_files or []
        model_utils.validate(
            archive_root_path=Path("."), model_file_path=model_file_path, artifact_files=artifact_files
        )
        all_file_paths = [*[Path(file) for file_str in artifact_files for file in glob.glob(file_str)], model_file_path]

        cuda_device = InferenceDeviceHardware.CPU
        if torch.cuda.is_available():
            device_count = torch.cuda.device_count()
            if device_count > 0:
                cuda_device = "cuda:0"

        with tempfile.TemporaryDirectory() as tmpdir:
            for file in all_file_paths:
                os.makedirs(Path(tmpdir) / os.path.dirname(file), exist_ok=True)
                shutil.copy(file, Path(tmpdir) / file)

            provided_keys = mock_inputs.keys()
            required_keys = {field.name for field in self.input_schema}

            if len(provided_keys - required_keys) != 0:
                msg = f"Input keys: {provided_keys - required_keys} are not in schema defined in ModelConfig"
                raise TectonValidationError(msg)

            if len(required_keys - provided_keys) != 0:
                msg = f"ModelConfig contains fields {required_keys - provided_keys} which are not provided in inputs"
                raise TectonValidationError(msg)

            input_columns = [Column(name=field.name, dtype=field.dtype.tecton_type) for field in self.input_schema]
            output_column = Column.from_proto(type_utils.to_column(self.output_schema))
            batch = pyarrow.RecordBatch.from_pydict(mock_inputs)

            model_container = CustomModelContainer(data_dir=Path(tmpdir), model_file_path=model_file_path)
            with torch.device(cuda_device):
                model_container.load()

            record_batch = custom_model_inference(
                batch=batch,
                model=model_container,
                input_columns=input_columns,
                output_column=output_column,
                model_input_schema=input_columns,
                cuda_device=cuda_device,
            )
            return record_batch.to_pandas()

    def register(self):
        """Register a Model Locally so that it can be used in Local Feature Views."""
        if self.name in _LOCAL_CUSTOM_MODELS:
            print(f"`ModelConfig` with name {self.name} already exists in context, will be replaced.")
        _LOCAL_CUSTOM_MODELS[self.name] = self
        print(f"Successfully registered model: {self.name} locally.")



from tecton_proto.args import data_source_config__client_pb2 as _data_source_config__client_pb2
from tecton_proto.args import transformation__client_pb2 as _transformation__client_pb2
from tecton_proto.args import user_defined_function__client_pb2 as _user_defined_function__client_pb2
from tecton_proto.common import schema__client_pb2 as _schema__client_pb2
from tecton_proto.common import spark_schema__client_pb2 as _spark_schema__client_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class KafkaDSConfig(_message.Message):
    __slots__ = ["bootstrap_servers", "security_protocol", "ssl_keystore_location", "ssl_keystore_password_secret_id", "ssl_truststore_location", "ssl_truststore_password_secret_id", "topics"]
    BOOTSTRAP_SERVERS_FIELD_NUMBER: ClassVar[int]
    SECURITY_PROTOCOL_FIELD_NUMBER: ClassVar[int]
    SSL_KEYSTORE_LOCATION_FIELD_NUMBER: ClassVar[int]
    SSL_KEYSTORE_PASSWORD_SECRET_ID_FIELD_NUMBER: ClassVar[int]
    SSL_TRUSTSTORE_LOCATION_FIELD_NUMBER: ClassVar[int]
    SSL_TRUSTSTORE_PASSWORD_SECRET_ID_FIELD_NUMBER: ClassVar[int]
    TOPICS_FIELD_NUMBER: ClassVar[int]
    bootstrap_servers: str
    security_protocol: str
    ssl_keystore_location: str
    ssl_keystore_password_secret_id: str
    ssl_truststore_location: str
    ssl_truststore_password_secret_id: str
    topics: str
    def __init__(self, bootstrap_servers: Optional[str] = ..., topics: Optional[str] = ..., ssl_keystore_location: Optional[str] = ..., ssl_keystore_password_secret_id: Optional[str] = ..., ssl_truststore_location: Optional[str] = ..., ssl_truststore_password_secret_id: Optional[str] = ..., security_protocol: Optional[str] = ...) -> None: ...

class KinesisDSConfig(_message.Message):
    __slots__ = ["region", "stream_name"]
    REGION_FIELD_NUMBER: ClassVar[int]
    STREAM_NAME_FIELD_NUMBER: ClassVar[int]
    region: str
    stream_name: str
    def __init__(self, stream_name: Optional[str] = ..., region: Optional[str] = ...) -> None: ...

class PushDataSourceConfig(_message.Message):
    __slots__ = ["input_schema", "log_offline", "post_processor", "post_processor_mode"]
    INPUT_SCHEMA_FIELD_NUMBER: ClassVar[int]
    LOG_OFFLINE_FIELD_NUMBER: ClassVar[int]
    POST_PROCESSOR_FIELD_NUMBER: ClassVar[int]
    POST_PROCESSOR_MODE_FIELD_NUMBER: ClassVar[int]
    input_schema: _schema__client_pb2.Schema
    log_offline: bool
    post_processor: _user_defined_function__client_pb2.UserDefinedFunction
    post_processor_mode: _transformation__client_pb2.TransformationMode
    def __init__(self, log_offline: bool = ..., post_processor: Optional[Union[_user_defined_function__client_pb2.UserDefinedFunction, Mapping]] = ..., input_schema: Optional[Union[_schema__client_pb2.Schema, Mapping]] = ..., post_processor_mode: Optional[Union[_transformation__client_pb2.TransformationMode, str]] = ...) -> None: ...

class SparkStreamDataSourceFunction(_message.Message):
    __slots__ = ["function"]
    FUNCTION_FIELD_NUMBER: ClassVar[int]
    function: _user_defined_function__client_pb2.UserDefinedFunction
    def __init__(self, function: Optional[Union[_user_defined_function__client_pb2.UserDefinedFunction, Mapping]] = ...) -> None: ...

class StreamDataSource(_message.Message):
    __slots__ = ["deduplication_column_names", "kafka_data_source", "kinesis_data_source", "options", "push_source", "raw_stream_translator", "spark_data_source_function", "spark_schema", "stream_config", "time_column"]
    class Option(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        key: str
        value: str
        def __init__(self, key: Optional[str] = ..., value: Optional[str] = ...) -> None: ...
    DEDUPLICATION_COLUMN_NAMES_FIELD_NUMBER: ClassVar[int]
    KAFKA_DATA_SOURCE_FIELD_NUMBER: ClassVar[int]
    KINESIS_DATA_SOURCE_FIELD_NUMBER: ClassVar[int]
    OPTIONS_FIELD_NUMBER: ClassVar[int]
    PUSH_SOURCE_FIELD_NUMBER: ClassVar[int]
    RAW_STREAM_TRANSLATOR_FIELD_NUMBER: ClassVar[int]
    SPARK_DATA_SOURCE_FUNCTION_FIELD_NUMBER: ClassVar[int]
    SPARK_SCHEMA_FIELD_NUMBER: ClassVar[int]
    STREAM_CONFIG_FIELD_NUMBER: ClassVar[int]
    TIME_COLUMN_FIELD_NUMBER: ClassVar[int]
    deduplication_column_names: _containers.RepeatedScalarFieldContainer[str]
    kafka_data_source: KafkaDSConfig
    kinesis_data_source: KinesisDSConfig
    options: _containers.RepeatedCompositeFieldContainer[StreamDataSource.Option]
    push_source: PushDataSourceConfig
    raw_stream_translator: _user_defined_function__client_pb2.UserDefinedFunction
    spark_data_source_function: SparkStreamDataSourceFunction
    spark_schema: _spark_schema__client_pb2.SparkSchema
    stream_config: _data_source_config__client_pb2.StreamConfig
    time_column: str
    def __init__(self, kinesis_data_source: Optional[Union[KinesisDSConfig, Mapping]] = ..., kafka_data_source: Optional[Union[KafkaDSConfig, Mapping]] = ..., spark_data_source_function: Optional[Union[SparkStreamDataSourceFunction, Mapping]] = ..., push_source: Optional[Union[PushDataSourceConfig, Mapping]] = ..., spark_schema: Optional[Union[_spark_schema__client_pb2.SparkSchema, Mapping]] = ..., raw_stream_translator: Optional[Union[_user_defined_function__client_pb2.UserDefinedFunction, Mapping]] = ..., deduplication_column_names: Optional[Iterable[str]] = ..., time_column: Optional[str] = ..., stream_config: Optional[Union[_data_source_config__client_pb2.StreamConfig, Mapping]] = ..., options: Optional[Iterable[Union[StreamDataSource.Option, Mapping]]] = ...) -> None: ...



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar, Iterable, Mapping, Optional, Union

DESCRIPTOR: _descriptor.FileDescriptor

class MaterializationRolesAllowlists(_message.Message):
    __slots__ = ["workspace_configurations"]
    WORKSPACE_CONFIGURATIONS_FIELD_NUMBER: ClassVar[int]
    workspace_configurations: _containers.RepeatedCompositeFieldContainer[WorkspaceMaterializationRolesAllowlist]
    def __init__(self, workspace_configurations: Optional[Iterable[Union[WorkspaceMaterializationRolesAllowlist, Mapping]]] = ...) -> None: ...

class WorkspaceMaterializationRolesAllowlist(_message.Message):
    __slots__ = ["allowed_materialization_roles", "workspace_name"]
    ALLOWED_MATERIALIZATION_ROLES_FIELD_NUMBER: ClassVar[int]
    WORKSPACE_NAME_FIELD_NUMBER: ClassVar[int]
    allowed_materialization_roles: _containers.RepeatedScalarFieldContainer[str]
    workspace_name: str
    def __init__(self, workspace_name: Optional[str] = ..., allowed_materialization_roles: Optional[Iterable[str]] = ...) -> None: ...
